#!/bin/sh
#-----------------------------------------------------------------------
#
# Tivoli Enterprise Data Warehouse, v 1.1.1 
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#-----------------------------------------------------------------------
#
# File Information: src/install/bin/twh_app_patcher.sh, twh_core, twh_1.1_ode
#
# Revision: 1.8
#
# Date: 8/19/02 16:46:14
#-----------------------------------------------------------------------
#
# Primary Author: D. Ledden
#
#
# Desc:	This program will patch and re-configure the Tivoli
#		Warehouse WPack Application components.
#
# Command Line Args:
#	-v = Release Version
#	-h = Help
#	-r = Show return code messages
#	-d = Run in Debug
#	-l = List installed products and patches
#
#
# Required Config File:
#	twh_app_patcher.cfg
#
#
# Required Env Vars:
#	None.
#
# Return Codes:
#	* See display_return_code_messages function below
#
#
# Interpreter Required:
#	NT/2000 	= BASH
#	Unix		= SH (Bourne Shell)
#
#
# Message Types:
#	(I) - Informational
#	(W) - Warning
#	(F) - Fatal
#
# Install Steps Required Before this Script Executes:
#
#########################################################################
#
###############################################################
################ Custom Functions Section #####################
###############################################################


# Pulls in the generic shared routines
# Note return codes 3-70 reserved for the gen routines
include_gen_routines()
{
# Set context for generic routines
TWH_APP_PATCH_CONTEXT=$TRUE; export TWH_APP_PATCH_CONTEXT

# Bug on NT - fix the path back to be unix type path
gen_file_and_path=`echo $PROGRAM | sed 's/\\\\/\//g' `

gen_path=`dirname "$gen_file_and_path" `

# Confirm file
# Look local, then the path
if [ ! -r "$gen_path"/twh_gen_routines.sh ]; then
  unix_topdir=`echo "$TWH_TOPDIR" | sed 's/\\\\/\//g' `

  if [ -r "$unix_topdir"/install/bin/twh_gen_routines.sh ]; then
    gen_path="$unix_topdir/install/bin"
  else
    echo "      (F) Could not locate the twh_gen_routines.sh file."
    exit 71
  fi
fi

# pull in the generic routines
. "$gen_path"/twh_gen_routines.sh

return 0;
}


###


# Display Help
display_help()
{
DISPLAY_HELP_ACTIVE=$TRUE;

echo ""
echo "  Program:  $PROGNAME"
echo "  Desc:     This is the main utility to patch Tivoli Enterprise Data Warehouse"
echo "            (TEDW) Application Pack components."
echo ""
echo "  Usage:    $PROGNAME -c config_file [ options... ]"
echo ""
echo "  Options:"
echo "            -c, --config       = The required config file"
echo "            -v, --version      = Version Information"
echo "            -h, --help         = Help Inforamtion"
echo "            -r, --retcode      = Return Code messages Inforamtion"
echo "            -d, --debug        = Run with Debug turned on"
echo "            -l, --listversions = List installed products and patches"
echo ""
echo "  Config File:"
echo "            twh_app_install.cfg"
echo ""
echo "  Message Levels:"
echo "		  (I) - Informational"
echo "		  (W) - Warning"
echo "		  (F) - Fatal"
echo ""

return 0
}

###


# Display return code messages
display_return_code_messages()
{
echo ""
echo "    Return code messages for this program:"
echo "    ======================================"

gr_display_return_codes;


echo ""
echo "    Application Install specific errors:"
echo "    255 = Application Afterscript Error"
echo ""

return 0;
}


###


# 
set_private_program_vars()
{
CORE_LOGDIR="$TWH_TOPDIR/install/logs"

CORE_PATCH_VERSION_FILE="$TWH_TOPDIR/.installed/.twh_core_v${TWH_VERSION}.marker"

PATCH_LOGS_DIR="$TWH_TOPDIR/install/logs/patch_${TWH_VERSION}"

# Must launch setup.exe from interp dir in patch tree
PATCH_TOPDIR=`echo "$PATCH_MEDIA_DIR"`;  export PATCH_TOPDIR
PATCH_GENERIC_DIR="$PATCH_TOPDIR/generic"

interp_lower=`echo $INTERP | tr '[A-Z]' '[a-z]' `
PATCH_INTERP_DIR="$PATCH_TOPDIR/$interp_lower"



return 0
}


###


#
display_warning()
{
echo "    ============================================================="
echo "                        WARNING!!!"
echo "    You should ensure you have completed all required manual"
echo "    backups of all Databases and product directories"
echo "    associated with the TEDW product.  These procedures are defined"
echo "    in the readme.txt file provided with the Application patch."
echo "    Failure to do so may result in the inability to restore"
echo "    this product in the state it was in prior to this patch."
echo ""
echo "    You can Contol-C to exit"
echo "    ============================================================="
echo ""
if [ "$INST_LOGDIR" != "" ]; then
  INST_LOGDIR=`echo "$INST_LOGDIR" | sed 's/\\\\/\//g' `	# Fix nt
  echo "  << Log for this patch run found at ($INST_LOGDIR) >>"
fi
echo ""


#sleep 10

return 0;
}

###


validate_basic_install()
{
echo "==> Checking for basic install info"
if [ ! -r "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate the Product Top directory ($TWH_TOPDIR)"
  echo "          Exiting..." 
  echo ""
  exit 72
fi


echo "      (I) Done checking for basic installation."
return 0;
}


###


# List the installed products and patches
#
exec_list_installed()
{
tmp_marker_file="$TMPDIR"/tmp_marker_list.txt
prod_count=0
prod_flag=0
date_val="01/01/2000"

echo "==> Listing the installed product, patch, and E-fix versions for the TEDW Core product."
echo ""


# Get the 1.1.0 product marker
rm -f "$tmp_marker_file"
cur_dir=`pwd`
cd "$TWH_TOPDIR"
ls -l ".twh_core_v1.1.0.marker" 2> /dev/null > "$tmp_marker_file";
# Get any other product markers

if [ -d "$TWH_TOPDIR"/.installed ]; then
  cd "$TWH_TOPDIR"/.installed
  ls -l ".twh_core_v*.*.*.marker" 2> /dev/null >> "$tmp_marker_file";
  cd "$cur_dir"
fi

echo ""
echo "    You have the following Core Product versions installed:"
echo ""
echo "          ==============="
echo ""

# Are any installed
if [ ! -s "$tmp_marker_file" ]; then
  echo "      (F) No Core installation versions detected on this system"
  echo "          This may reflect a failed Core installation attempt."
  echo "          Exiting..."
  echo ""
  exit 76
fi


# read the file to grab the versions
while :
  do
    read list_rec
    if [ "$list_rec" = "" ]; then
        break
    fi

    v_file=`echo $list_rec | awk '{print $9}' `
    file_date=`echo $list_rec | awk '{print $6,$7,$8}' `

    # Fetch the version info
    get_version_info "$v_file";


    # Print out list
    if [ "$HAVE_EFIX" = "$TRUE" ]; then
      echo "             Efix:      $TWH_EFIX_VER  installed on $file_date"
      continue
    fi

    if [ $TWH_PATCH_VER -eq 0 ]; then
      if [ $prod_flag -eq 1 ]; then
        echo "";
        prod_flag=0
      fi
      echo "          Product:  $TWH_VERSION_VAL  installed on $file_date"
      prod_flag=1
	prod_count=`expr $prod_count + 1 `		# bump count
    else
      echo "            Patch:  $TWH_VERSION_VAL  installed on $file_date"
    fi


  done < "$tmp_marker_file"
rm -f "$tmp_marker_file"


if [ $prod_count -eq 0 ]; then
  echo "          *  <None found>"
fi

echo ""
echo "          ==============="
echo ""
echo ""



echo ""
echo "    You have the following Warehouse Pack Product versions installed:"
echo ""
echo "          ==============="


list_apps_installed;

echo ""
echo "          ==============="
echo ""
echo ""




echo "      (I) Done listing installed versions"
echo "          Exiting..."
exit 0

return 0;
}


###


##########################################################
# Get the release, maintenence release, and patch versions
# 	$1 = the version marker file
#	Output:
#		TWH_VERSION_VAL = The entire version
#		TWH_RELEASE_VER = The release digit
#		TWH_MAINT_VER = The maintenence digit
#		TWH_PATCH_VER = The patch digit
#
# Core versions will be
#	.twh_core_v1.2.3.marker
#                r.m.p
#     Where
#	r.	= Release digit
#	.m	= Maintenance digit
#	.p	= Patch digit
#
# Efix version format will be:
#	.twh_core_v1.1.1@efix4.marker
#                          _e
#	Where
#	_e	= Efix number
##########################################################
get_version_info()
{
HAVE_EFIX="$FALSE"
marker_file="$1"
TWH_RELEASE_VER=0
TWH_MAINT_VER=0
TWH_PATCH_VER=0
TWH_EFIX_VER=0

if [ -z "$marker_file" ]; then
  echo "      (F) Marker file not supplied to function get_version_info"
  exit 77
fi

# get the release, maintenance release, and patch numbers
tmp_version_val=`echo "$marker_file" | sed "s/\.twh_core_v//" `
tmp_version_val=`echo "$tmp_version_val" | sed "s/\.marker//" `

# strip possible efix val
TWH_VERSION_VAL=` echo "$tmp_version_val" | sed "s/@efix.*//" `


# Release
TWH_RELEASE_VER=`echo "$TWH_VERSION_VAL" | sed "s/\..*//g" `

# Maintenance version
TWH_MAINT_VER=`echo "$TWH_VERSION_VAL" |  sed "s/^[0-9]*//" | sed "s/[0-9]*$//" |  sed "s/\.//g" `


# Patch version
TWH_PATCH_VER=`echo "$TWH_VERSION_VAL" | sed "s/.*\.//g" | sed "s/\@efix.*//g" `


# Efix version
blah=` echo "$tmp_version_val" | grep -i "@efix" `
if [ $? -eq 0 ]; then
  TWH_EFIX_VER=` echo "$tmp_version_val" | sed "s/.*@efix//" `
  HAVE_EFIX="$TRUE"
fi


return 0;
}


###


#
list_apps_installed()
{
app_list_file="$TMPDIR"/apps_list.txt
#echo ""
#echo "          Applications detected on this system are:"
#echo ""

# Fetch the list of apps
ls -d "$TWH_TOPDIR"/apps/* 2> /dev/null > "$app_list_file";


# for efix loop
cur_dir1=`pwd`


loop_count=1
# List what is out there
while :
  do
    read app
    if [ "$app" = "" ]; then
      if [ $loop_count -eq 1 ]; then
        echo "            < No applications found on this system >"
      fi

      break;
    fi

    app_name=`basename "$app" `
    app_lookup="$app_name"
    app_name=`echo $app_name | tr '[a-z]' '[A-Z]'`

    echo "          $loop_count) $app_name"

    # Was is completely installed?
    blah=`ls "$app"/.*.marker 2> /dev/null `
    if [ $? -ne 0 ]; then
      echo "                     (W) Application may not have fully installed"
    fi


    check_version "$app_lookup" "$APP_REL_VERSION" "$TRUE";
    echo "             Latest version:    $VERSION_INSTALLED"

    echo "             E-Fixes Added:"
    # Any Efixes?
    if [ -r "$TWH_TOPDIR"/apps/${app_lookup}/.installed ]; then
      cd "$TWH_TOPDIR/apps/${app_lookup}/.installed"
      rm -f "$TMPDIR"/efix_list.tmp

      ls ".*@efix*.marker" 2> /dev/null > "$TMPDIR"/efix_list.tmp
      if [ $? -ne 0 ]; then
        echo "                                <None>"
      else
        while :
          do

          read efix_ver
          if [ "$efix_ver" = "" ]; then
            break;
          fi

          # Clean up version
          efix_ver=`echo "$efix_ver" | sed "s/.twh_${app_lookup}_v//" | sed "s/.marker//" `
          echo "                                $efix_ver"
        done < "$TMPDIR"/efix_list.tmp
      fi
    else
      echo "                                <None>"      
    fi

  echo ""
  loop_count=`expr $loop_count + 1 `
  done < "$app_list_file"


# Go back where we were
cd "$cur_dir1"

# Remove the junk
rm -f "$app_list_file"

return 0;
}


###


#
validate_app_path()
{
echo "==> Checking for Warehouse Pack..."

# validate the app directory
if [ ! -d "$APP_MEDIA_DIR" ]; then
  echo "      (F) Unable to find or read application directory of"
  echo "          ($APP_MEDIA_DIR)."
  echo "          Exiting..."
  exit 78
fi

#Validate this is an application directory 
if [ ! -r "$APP_MEDIA_DIR"/twh_install_props.cfg ]; then
  echo "      (F) Application media directory provided is not valid($APP_MEDIA_DIR)."
  echo "          It is missing the twh_install_props.cfg file."
  echo "          Exiting..."
  exit 79
fi

# Set the application
APP_TO_INSTALL=`basename "$APP_MEDIA_DIR" `; export APP_TO_INSTALL

# Check the props file for that value (should all be lower case)
test_ava_val=`cat "$APP_MEDIA_DIR"/twh_install_props.cfg | grep "AVA_CODE=" | sed 's/AVA_CODE=//' `

# convert to lower case
APP_TO_INSTALL=`echo $APP_TO_INSTALL | tr '[A-Z]' '[a-z]' `
test_ava_val=`echo $test_ava_val | tr '[A-Z]' '[a-z]' `

if [ "$APP_TO_INSTALL" != "$test_ava_val" ]; then
  echo "      (F) Invalid AVA code value in twh_install_props.cfg file"
  echo "          compared to the install directory."
  exit 80
else
  echo ""
  echo "     (I) Application detected."
  echo "           Warehouse Pack: $APP_TO_INSTALL"
  echo "           Located at:     $APP_MEDIA_DIR."
  echo ""
fi

return 0;
}


###


#
validate_core_comp_list_file()
{
# Validate all properties are here
if [ -z "$TWH_CORE_INSTALL_TYPE" ]; then
  echo "      (F) Error.  Property TWH_CORE_INSTALL_TYPE not found in config file"
  exit 81
elif [ -z "$TWH_CTRL_COMP" ]; then
  echo "      (F) Error.  Property TWH_CTRL_COMP not found in config file"
  exit 81
elif [ -z "$TWH_CTRL_HOST" ]; then
  echo "      (F) Error.  Property TWH_CTRL_HOST not found in config file"
  exit 81
elif [ -z "$TWH_CDW_HOST" ]; then
  echo "      (F) Error.  Property TWH_CDW_HOST not found in config file"
  exit 81
elif [ -z "$TWH_MART_HOST" ]; then
  echo "      (F) Error.  Property TWH_MART_HOST not found in config file"
  exit 81
elif [ -z "$TWH_RPI_HOST" ]; then
  echo "      (F) Error.  Property TWH_RPI_HOST not found in config file"
  exit 81
elif [ -z "$TWH_CDW_COMP" ]; then
  echo "      (F) Error.  Property TWH_CDW_COMP not found in config file"
  exit 81
elif [ -z "$TWH_MART_COMP" ]; then
  echo "      (F) Error.  Property TWH_MART_COMP not found in config file"
  exit 81
elif [ -z "$TWH_RPI_COMP" ]; then
  echo "      (F) Error.  Property TWH_RPI_COMP not found in config file"
  exit 81
elif [ -z "$TWH_DB2_NODE_MAJIC" ]; then
  echo "      (F) Error.  Property TWH_DB2_NODE_MAJIC not found in config file"
  exit 81
elif [ -z "$TWH_CTRL_NODE_ALIAS" ]; then
  echo "      (F) Error.  Property TWH_CTRL_NODE_ALIAS not found in config file"
  exit 81
elif [ -z "$TWH_CDW_NODE_ALIAS" ]; then
  echo "      (F) Error.  Property TWH_CDW_NODE_ALIAS not found in config file"
  exit 81
elif [ -z "$TWH_MART_NODE_ALIAS" ]; then
  echo "      (F) Error.  Property TWH_MART_NODE_ALIAS not found in config file"
  echo "          Ensure a carriage return is not missing from the last line in the config file"
  exit 81
fi


return 0;
}


###


#  Read the supplied core component list file
#  This file will tell us what type of core install was
#  selected and some properties if they did a Custom install
read_core_comp_list_file()
{
echo "==> Reading the Core Component List file located at ($TWH_TOPDIR/install/logs)."


# is it there
if [ ! -r "$TWH_CORE_COMP_FILE" ]; then
  echo "      (F) Could not find or read core comp file"
  echo "          for application ($CUR_APP). File should be"
  echo "          ($TWH_CORE_COMP_FILE)."
  exit 82
fi


# Open our config file
exec < "$TWH_CORE_COMP_FILE"


# Parse the core comp file
while :
  do
  # Read a line
  read cur_line

  # Did we hit the end of the file?
  if [ $? -ne 0 ]; then break; fi

  # clean whitespace and look for comment char
  cleaned_line=`echo "$cur_line" | sed "s/        //g" `

  # On Windows, do this to avoid ctrl M issues
  if [ "$INTERP" = "$WIN" ]; then
    blah=`echo "$cleaned_line" | grep -v '[a-zA-Z].*' `
    if [ $? -eq 0 ]; then continue; fi
  fi

  # Goto next line if this is a comment
  echo "$cleaned_line" | grep -i "^#" > /dev/null
  if [ $? -eq 0  -o ! "$cleaned_line" ]; then
    continue
  fi

  # Now parse for properties and validate them
  echo "$cleaned_line" | grep "=" > /dev/null
  if [ ! $? -eq 0 ]; then
    gr_print_conf_read_error "$cleaned_line" ;
    exit 83
  fi

  # Find the property tag and value and upcase the tag
  property_line=`echo "$cleaned_line" | sed "s/#.*//" `
  prop_tag=`echo "$property_line" | sed "s/=.*//" | tr '[a-z]' '[A-Z]' `
  prop_val=`echo "$property_line" | sed "s/.*=//" `

  # Ensure they supplied something
  if [ -z "$prop_val" ]; then
    echo "      (F) No value found for property ($prop_tag)."
    exit 84
  fi

  # Now define the property for later use
  case $prop_tag in

    #
    TWH_CORE_INSTALL_TYPE)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CORE_INSTALL_TYPE="$prop_val"
        echo "      (I) Core installation type was ($TWH_CORE_INSTALL_TYPE)"
        uppr_val=`echo "$TWH_CORE_INSTALL_TYPE" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TWH_CUSTOM" ]; then 
          CUSTOM_INSTALL=$TRUE
        fi
        export CUSTOM_INSTALL
        ;;

    #
    TWH_CTRL_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_COMP="$prop_val"
        uppr_val=`echo "$TWH_CTRL_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          ADMIN_CHECKED=$TRUE
        fi
        export ADMIN_CHECKED
        ;;

    #
    TWH_CTRL_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_HOST="$prop_val"
        export TWH_CTRL_HOST
        ;;

    #
    TWH_CDW_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_HOST="$prop_val"
        export TWH_CDW_HOST
        ;;

    #
    TWH_MART_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_MART_HOST="$prop_val"
        export TWH_MART_HOST
        ;;

    #
    TWH_RPI_HOST)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_RPI_HOST="$prop_val"
        export TWH_RPI_HOST
        ;;

    #
    TWH_CDW_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_COMP="$prop_val"
        uppr_val=`echo "$TWH_CDW_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          CDW_CHECKED=$TRUE
        fi
        export CDW_CHECKED
        ;;
    #
    TWH_MART_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_MART_COMP="$prop_val"
        uppr_val=`echo "$TWH_MART_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          MART_CHECKED=$TRUE
        fi
        export MART_CHECKED
        ;;
    #

    TWH_RPI_COMP)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_RPI_COMP="$prop_val"
        uppr_val=`echo "$TWH_RPI_COMP" | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then 
          RPI_CHECKED=$TRUE
        fi
        export RPI_CHECKED
        ;;

    TWH_DB2_NODE_MAJIC)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_DB2_NODE_MAJIC="$prop_val"
        export TWH_DB2_NODE_MAJIC
        ;;

    TWH_CTRL_NODE_ALIAS)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CTRL_NODE_ALIAS="$prop_val"
        export TWH_CTRL_NODE_ALIAS
        ;;

    TWH_CDW_NODE_ALIAS)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CDW_NODE_ALIAS="$prop_val"
        export TWH_CDW_NODE_ALIAS
        ;;

    TWH_MART_NODE_ALIAS)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_MART_NODE_ALIAS="$prop_val"
        export TWH_MART_NODE_ALIAS
        ;;

    # Default - is an error
    *)
        echo "      (F) Illegal option found."
        echo "          Illegal value is ($prop_tag)"
        exit 85
        ;;
    esac
  done


# Detect any missing properties
validate_core_comp_list_file;

echo "      (I) Completed reading Core Component List file"
return 0;
}


###


#
validate_app_props_file()
{
# Validate all properties are here
if [ -z "$APP_DISPLAY_NAME" ]; then
  echo "      (F) Error.  Property APP_DISPLAY_NAME not found in config file"
  exit 86
elif [ -z "$AVA_CODE" ]; then
  echo "      (F) Error.  Property AVA_CODE not found in config file"
  exit 86
elif [ -z "$TWH_CORE_PREREQ_VERSION" ]; then
  echo "      (F) Error.  Property TWH_CORE_PREREQ_VERSION not found in config file"
  exit 86
elif [ -z "$APP_REL_VERSION" ]; then
  echo "      (F) Error.  Property APP_REL_VERSION not found in config file"
  exit 86
elif [ -z "$PRE_SCRIPT" ]; then
  echo "      (F) Error.  Property PRE_SCRIPT not found in config file"
  exit 86
elif [ -z "$PRE_ETL_SCRIPT" ]; then
  echo "      (F) Error.  Property PRE_ETL_SCRIPT not found in config file"
  exit 86
elif [ -z "$AFTER_SCRIPT" ]; then
  echo "      (F) Error.  Property AFTER_SCRIPT not found in config file"
  exit 86
elif [ -z "$APP_DATAMART_SCHEMA" ]; then
  echo "      (F) Error.  Property APP_DATAMART_SCHEMA not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_AVA_NAME_1" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_AVA_NAME_1 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_VERSION_1" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_VERSION_1 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_AVA_NAME_2" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_AVA_NAME_2 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_VERSION_2" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_VERSION_2 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_AVA_NAME_3" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_AVA_NAME_3 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_VERSION_3" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_VERSION_3 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_AVA_NAME_4" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_AVA_NAME_4 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_VERSION_4" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_VERSION_4 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_AVA_NAME_5" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_AVA_NAME_5 not found in config file"
  exit 86
elif [ -z "$WPACK_PREREQ_VERSION_5" ]; then
  echo "      (F) Error.  Property WPACK_PREREQ_VERSION_5 not found in config file"
  exit 86
elif [ -z "$CDW_INCLUDED" ]; then
  echo "      (F) Error.  Property CDW_INCLUDED not found in config file"
  exit 86
elif [ -z "$DOCS_INCLUDED" ]; then
  echo "      (F) Error.  Property DOCS_INCLUDED not found in config file"
  exit 86
elif [ -z "$ETL_SQL_INCLUDED" ]; then
  echo "      (F) Error.  Property ETL_SQL_INCLUDED not found in config file"
  exit 86
elif [ -z "$ETL_TAG_INCLUDED" ]; then
  echo "      (F) Error.  Property ETL_TAG_INCLUDED not found in config file"
  exit 86
elif [ -z "$I18N_INCLUDED" ]; then
  echo "      (F) Error.  Property I18N_INCLUDED not found in config file"
  exit 86
elif [ -z "$MART_INCLUDED" ]; then
  echo "      (F) Error.  Property MART_INCLUDED not found in config file"
  exit 86
elif [ -z "$REPORTS_INCLUDED" ]; then
  echo "      (F) Error.  Property REPORTS_INCLUDED not found in config file"
  echo "          Ensure a carriage return is not missing from the last line in the config file"
  exit 86
fi

return 0;
}


###


#  Read the supplied application property file
#  	$1 = Current Application Description
#	$2 = Current application media directory
read_app_props_file()
{
CUR_APP="$1"
CUR_APP_DIR="$2"
echo "==> Reading the application property file"
echo "    ($CUR_APP_DIR/$APP_INSTALL_PROPS_FILE) for application ($CUR_APP)."


# is it there
if [ ! -r "$CUR_APP_DIR/$APP_INSTALL_PROPS_FILE" ]; then
  echo "      (F) Could not find or read application properties file"
  echo "          for application ($CUR_APP). File is"
  echo "          ($CUR_APP_DIR/$APP_INSTALL_PROPS_FILE)."
  exit 87
fi

# Open our config file
exec < "$CUR_APP_DIR/$APP_INSTALL_PROPS_FILE"



# Parse the app properties config file
while :
  do
  # Read the config file
  read cur_line

  # Did we hit the end of the file?
  if [ $? -ne 0 ]; then break; fi

  # clean whitespace and look for comment char
  cleaned_line=`echo "$cur_line" | sed "s/        //g" `

  # On Windows, do this to avoid ctrl M issues
  if [ "$INTERP" = "$WIN" ]; then
    blah=`echo "$cleaned_line" | grep -v '[a-zA-Z].*' `
    if [ $? -eq 0 ]; then continue; fi
  fi

  # Goto next line if this is a comment
  echo "$cleaned_line" | grep -i "^#" > /dev/null
  if [ $? -eq 0  -o ! "$cleaned_line" ]; then
    continue
  fi

  # Now parse for properties and validate them
  echo "$cleaned_line" | grep "=" > /dev/null
  if [ ! $? -eq 0 ]; then
    gr_print_conf_read_error "$cleaned_line" ;
    exit 88
  fi

  # Find the property tag and value and upcase the tag
  property_line=`echo "$cleaned_line" | sed "s/#.*//" `
  prop_tag=`echo "$property_line" | sed "s/=.*//" | tr '[a-z]' '[A-Z]' `
  prop_val=`echo "$property_line" | sed "s/.*=//" `

  # Ensure they supplied something
  if [ -z "$prop_val" ]; then
    echo "      (F) No value found for property ($prop_tag)."
    exit 89
  fi

  # Now define the property for later use
  case $prop_tag in

    #
    APP_DISPLAY_NAME)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APP_DISPLAY_NAME="$prop_val"
        export APP_DISPLAY_NAME
        echo "      (I) Application Product is ($APP_DISPLAY_NAME)"
        ;;

    #
    AVA_CODE)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        AVA_CODE="$prop_val"
        export AVA_CODE
        ;;

    #
    TWH_CORE_PREREQ_VERSION)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        TWH_CORE_PREREQ_VERSION="$prop_val"
        export TWH_CORE_PREREQ_VERSION
        ;;

    #
    APP_REL_VERSION)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APP_REL_VERSION="$prop_val"
        export APP_REL_VERSION
        ;;

    #
    PRE_SCRIPT)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PRE_SCRIPT="$prop_val"
        export PRE_SCRIPT
        ;;

    #
    PRE_ETL_SCRIPT)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        PRE_ETL_SCRIPT="$prop_val"
        export PRE_ETL_SCRIPT
        ;;

    #
    AFTER_SCRIPT)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        AFTER_SCRIPT="$prop_val"
        export AFTER_SCRIPT
        ;;

    #
    APP_DATAMART_SCHEMA)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        APP_DATAMART_SCHEMA="$prop_val"
        export APP_DATAMART_SCHEMA
        ;;

    #
    WPACK_PREREQ_AVA_NAME_1)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_AVA_NAME_1="$prop_val"
        export WPACK_PREREQ_AVA_NAME_1
        ;;

    #
    WPACK_PREREQ_VERSION_1)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_VERSION_1="$prop_val"
        export WPACK_PREREQ_VERSION_1
        ;;

    # These next 4 are optional values
    WPACK_PREREQ_AVA_NAME_2)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_AVA_NAME_2="$prop_val"
        export WPACK_PREREQ_AVA_NAME_2
        ;;

    #
    WPACK_PREREQ_VERSION_2)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_VERSION_2="$prop_val"
        export WPACK_PREREQ_VERSION_2
        ;;

    #
    WPACK_PREREQ_AVA_NAME_3)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_AVA_NAME_3="$prop_val"
        export WPACK_PREREQ_AVA_NAME_3
        ;;

    #
    WPACK_PREREQ_VERSION_3)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_VERSION_3="$prop_val"
        export WPACK_PREREQ_VERSION_3
        ;;

    #
    WPACK_PREREQ_AVA_NAME_4)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_AVA_NAME_4="$prop_val"
        export WPACK_PREREQ_AVA_NAME_4
        ;;

    #
    WPACK_PREREQ_VERSION_4)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_VERSION_4="$prop_val"
        export WPACK_PREREQ_VERSION_4
        ;;

    #
    WPACK_PREREQ_AVA_NAME_5)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_AVA_NAME_5="$prop_val"
        export WPACK_PREREQ_AVA_NAME_5
        ;;

    #
    WPACK_PREREQ_VERSION_5)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        WPACK_PREREQ_VERSION_5="$prop_val"
        export WPACK_PREREQ_VERSION_5
        ;;


    # 
    CDW_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          CDW_INCLUDED="TRUE"
        else
          CDW_INCLUDED="FALSE"
        fi
        export CDW_INCLUDED
        ;;

    # 
    DOCS_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          DOCS_INCLUDED="TRUE"
        else
          DOCS_INCLUDED="FALSE"
        fi
        export DOCS_INCLUDED
        ;;

    # 
    ETL_SQL_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          ETL_SQL_INCLUDED="TRUE"
        else
          ETL_SQL_INCLUDED="FALSE"
        fi
        export ETL_SQL_INCLUDED
        ;;

    # 
    ETL_TAG_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          ETL_TAG_INCLUDED="TRUE"
        else
          ETL_TAG_INCLUDED="FALSE"
        fi
        export ETL_TAG_INCLUDED
        ;;

    # 
    I18N_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          I18N_INCLUDED="TRUE"
        else
          I18N_INCLUDED="FALSE"
        fi
        export I18N_INCLUDED
        ;;

    # 
    MART_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          MART_INCLUDED="TRUE"
        else
          MART_INCLUDED="FALSE"
        fi
        export MART_INCLUDED
        ;;

    # 
    REPORTS_INCLUDED)
        if [ $DEBUG ]; then echo "$prop_tag found"; fi
        uppr_val=`echo $prop_val | tr '[a-z]' '[A-Z]'`
        if [ "$uppr_val" = "TRUE" ]; then
          REPORTS_INCLUDED="TRUE"
        else
          REPORTS_INCLUDED="FALSE"
        fi
        export REPORTS_INCLUDED
        ;;

    # Default - is an error
    *)
        echo "      (F) Illegal option found."
        echo "          Illegal value is ($prop_tag)"
        exit 90
        ;;
    esac
  done


# Detect any missing properties
validate_app_props_file;


# inform on package components to install
echo "      (I) Package components to install are:"
echo "            ================================"

none_found=$TRUE
if [ "$CDW_INCLUDED" = "TRUE" ]; then
  echo "            CDW"
  none_found=$FALSE
fi
if [ "$DOCS_INCLUDED" = "TRUE" ]; then
  echo "            DOCS"
  none_found=$FALSE
fi
if [ "$ETL_TAG_INCLUDED" = "TRUE" ]; then
  echo "            ETL/TAG"
  none_found=$FALSE
fi
if [ "$ETL_SQL_INCLUDED" = "TRUE" ]; then
  echo "            ETL/SQL"
  none_found=$FALSE
fi
if [ "$I18N_INCLUDED" = "TRUE" ]; then
  echo "            I18N"
  none_found=$FALSE
fi
if [ "$MART_INCLUDED" = "TRUE" ]; then
  echo "            MART"
  none_found=$FALSE
fi
if [ "$REPORTS_INCLUDED" = "TRUE" ]; then
  echo "            REPORTS"
  none_found=$FALSE
fi


# Set to upper
pre=`echo "$PRE_SCRIPT" | tr '[a-z]' '[A-Z]' `
pre_etl=`echo "$PRE_ETL_SCRIPT" | tr '[a-z]' '[A-Z]' `
after=`echo "$AFTER_SCRIPT" | tr '[a-z]' '[A-Z]' `


# Error here?
if [ $none_found -eq $TRUE -a "$pre" = "NULL" -a "$pre_etl" = "NULL" -a "$after" = "NULL" ]; then
  echo "             <None>"
  echo ""
  echo "      (F) No work defined in the twh_install_props.cfg file for this patch."
  echo "          You need to supply at least a pre or afterscript, or set one of the"
  echo "          components to install to true."
  echo "          Exiting..."
  exit 1
elif [ $none_found -eq $TRUE ]; then
  echo "             <None>"
  echo "             Only externally hooked scripts will be run (like pre or after scripts)"
fi
echo ""


echo "      (I) Completed reading Application Properties File"
return 0;
}


###


# Exported values out of this function:
# 	VERSION_MAJOR
# 	VERSION_MINOR
# 	VERSION_MAINT
# 	VERSION_FIXPK
# 	VERSION_REL = (GREATER, EQUAL, LESS)
# This function will determine the relationship between the installing program
# and the highest installed level.  Context for relationship is "app is (greater, equal,
# or less ) then the highest installed software version
check_version()
{
    app_name=$1
    app_ver=$2
    ignore_display="$3"			# Optional

    markers=""
    VERSION_MAJOR=0
    VERSION_MINOR=0
    VERSION_MAINT=0
    VERSION_FIXPK=0
    VERSION_REL=""
    MFDIR="$TWH_TOPDIR";
    VERSION_INSTALLED="0.0.0.0"

    if [ "$app_name" != "core" ]; then
        MFDIR="$TWH_TOPDIR"/apps/$app_name
    fi


    # Find latest installed level of app_name given.

    ls "$MFDIR"/.twh_$app_name_*.marker 2> /dev/null > "$TMPDIR/version.dat"
    ls "$MFDIR"/.installed/.twh_$app_name_*.marker 2> /dev/null >> "$TMPDIR"/version.dat

    while :
      do

	read i
      if [ "$i" = "" ]; then
        break;
      fi

	marker=`basename "$i"`

	# Skip this dummy record
	if [ "$marker" = ".twh_core_custom.marker" ]; then
        continue;
	fi 

	# Skip E-Fix values.  No dependency on E-Fixes
	blah=`echo $marker | grep @efix 2>&1 `
	if [ $? -eq 0 ]; then
        continue;
	fi


	# Convert to lower for tests
	marker=`echo $marker | tr '[A-Z]' '[a-z]' `

      # Strip marker file of everything but the version info.
	VERSION_INSTALLED=`echo $marker | sed "s/\.twh_${app_name}_v//g" | sed "s/\.marker//g" `
	VERSION_INSTALLED=`echo $VERSION_INSTALLED | cut -d"@" -f1`

      MF_MAJOR=`echo $VERSION_INSTALLED | cut -d"." -f1`
	MF_MINOR=`echo $VERSION_INSTALLED | cut -d"." -f2`
	MF_MAINT=`echo $VERSION_INSTALLED | cut -d"." -f3`
	MF_FIXPK=`echo $VERSION_INSTALLED | cut -d"." -f4`
      
  
        # Windows hack for versions that do not have the 4th digit in the version 
        if [ "$MF_FIXPK" = "$VERSION_INSTALLED" ]; then
            MF_FIXPK=0
        fi


	if [ $MF_MAJOR -gt $VERSION_MAJOR ]; then
            VERSION_MAJOR=$MF_MAJOR
	    VERSION_MINOR=$MF_MINOR
	    VERSION_MAINT=$MF_MAINT
            if [ "$MF_FIXPK" != "" ]; then
	        VERSION_FIXPK=$MF_FIXPK
            else
	        VERSION_FIXPK=0
            fi
        elif [ $MF_MAJOR -eq $VERSION_MAJOR ]; then
            if [ $MF_MINOR -gt $VERSION_MINOR ]; then
    	        VERSION_MINOR=$MF_MINOR
	        VERSION_MAINT=$MF_MAINT
                if [ "$MF_FIXPK" != "" ]; then
	            VERSION_FIXPK=$MF_FIXPK
                else
	            VERSION_FIXPK=0
                fi
            elif [ $MF_MINOR -eq $VERSION_MINOR ]; then
                if [ $MF_MAINT -gt $VERSION_MAINT ]; then
  	            VERSION_MAINT=$MF_MAINT
                    if [ "$MF_FIXPK" != "" ]; then
	                VERSION_FIXPK=$MF_FIXPK
                    else
	                VERSION_FIXPK=0
                    fi
                elif [ $MF_MAINT -eq $VERSION_MAINT ]; then
                    if [ "$MF_FIXPK" != "" ]; then
                        if [ $MF_FIXPK -gt $VERSION_FIXPK ]; then
      	                    VERSION_FIXPK=$MF_FIXPK
			fi
                    fi
                fi
            fi
        fi
    done < "$TMPDIR"/version.dat
    rm -f "$TMPDIR"/version.dat


    # Set highest version
    VERSION_INSTALLED="${VERSION_MAJOR}.${VERSION_MINOR}.${VERSION_MAINT}.${VERSION_FIXPK}"
    if [ "$ignore_display" = "" ]; then 
      echo "      (I) Highest version installed is ($VERSION_INSTALLED)"
    fi

    if [ "$app_ver" != "" ]; then
	  breakdown_version_levels "$app_ver";

        APP_MAJOR=$MAJOR
        APP_MINOR=$MINOR
        APP_MAINT=$MAINT
        APP_FIXPK=$FIXPK

	# Fourth digit was optional
	if [ "$APP_FIXPK" = "" ]; then APP_FIXPK=0; fi

	# Set Equal, greater, less
      VERSION_REL="LESS"
	if [ $APP_MAJOR -gt $VERSION_MAJOR ]; then
            VERSION_REL="GREATER"
        elif [ $APP_MAJOR -eq $VERSION_MAJOR ]; then
            if [ $APP_MINOR -gt $VERSION_MINOR ]; then
                VERSION_REL="GREATER"
            elif [ $APP_MINOR -eq $VERSION_MINOR ]; then
                if [ $APP_MAINT -gt $VERSION_MAINT ]; then
                    VERSION_REL="GREATER"
                elif [ $APP_MAINT -eq $VERSION_MAINT ]; then
                    if [ $APP_FIXPK -gt $VERSION_FIXPK ]; then
                        VERSION_REL="GREATER"
                    elif [ $APP_FIXPK -eq $VERSION_FIXPK ]; then
                        VERSION_REL="EQUAL"
                    fi
                fi
            fi
        fi
    fi


return 0;
}


###


#
breakdown_version_levels()
{
ver_in="$1"

MAJOR=`echo $ver_in | cut -d"." -f1`
MINOR=`echo $ver_in | cut -d"." -f2`
MAINT=`echo $ver_in | cut -d"." -f3`
FIXPK=`echo $ver_in | cut -d"." -f4`

#Fourth digit was optional
if [ "$FIXPK" = "" ]; then FIXPK=0; fi


return 0;
}


###


#
check_core_tools_installed()
{
echo "==> Checking for Core Tools Version"


breakdown_version_levels "$TWH_CORE_PREREQ_VERSION";
check_version "core" "$TWH_CORE_PREREQ_VERSION";


if [ "$VERSION_REL" = "LESS" -o "$VERSION_REL" = "EQUAL" ]; then

  echo "      (I) Base version validated."
else
  echo "      (F) The Mininum TWH Core Tools Version Required"
  echo "          ($TWH_CORE_PREREQ_VERSION) was not detected on this system or"
  echo "          was not successfully installed."
  echo "          You cannot install this application without it."
  echo "          Exiting..."
  exit 91
fi


echo "      (I) Core Tools version of ($TWH_CORE_PREREQ_VERSION) is acceptable to install"
return 0;
}


###


#
set_app_program_vars()
{
APP_TOPDIR="$TWH_TOPDIR/apps/$AVA_CODE"

# Set APP_WORK_DIR for each application
app_rel_ver=`ls "$TWH_TOPDIR"/apps/$AVA_CODE | grep "^v" ` 
APP_WORK_DIR="$APP_TOPDIR/$app_rel_ver"; export APP_WORK_DIR


# See if we can setup a backup dir
check_version "$AVA_CODE" "$APP_REL_VERSION";
APP_PREV_VERSION=$VERSION_INSTALLED

# Patch App backup directory
if [ ! -r "$TWH_TOPDIR"/apps_backups/$AVA_CODE/${APP_PREV_VERSION}_backup ]; then 
  mkdir -p "$TWH_TOPDIR/apps_backups/$AVA_CODE/${APP_PREV_VERSION}_backup"
  chmod 755 "$TWH_TOPDIR/apps_backups/$AVA_CODE/${APP_PREV_VERSION}_backup"
fi
APP_BACKUP_DIR="$TWH_TOPDIR/apps_backups/$AVA_CODE/${APP_PREV_VERSION}_backup"


# Set some additional env vars
APP_VER_TAG_FILE="$APP_TOPDIR/.installed/.twh_${AVA_CODE}_v${APP_REL_VERSION}.marker"

APP_PATCH_FILE_EXT="@patch_$APP_REL_VERSION"
APP_EXPORT_DIR="$APP_WORK_DIR/etl/export"

return 0
}


###


# Prevent an app from reinstalling - if installed, go to next one.
validate_app_rel_version()
{
echo "==> Validating the application release version so this patch can be installed."


breakdown_version_levels "$APP_REL_VERSION";
check_version "$AVA_CODE" "$APP_REL_VERSION";

# Make 2 values public (afterscript use)
APP_LATEST_FIXPACK_VER=$VERSION_FIXPK; export APP_LATEST_FIXPACK_VER
APP_FIXPACK_VER=$FIXPK; export APP_FIXPACK_VER


if [ "$MAJOR" = "$VERSION_MAJOR" -a "$MINOR" = "$VERSION_MINOR" -a "$MAINT" = "$VERSION_MAINT" ]; then
  echo "      (I) Base version is acceptable to install"
else
  echo "      (F) Base Warehouse pack for product ($AVA_CODE) not found or was not"
  echo "          successfully installed."
  echo "          Did not detect successful installation of version (${MAJOR}.${MINOR}.${MAINT}.{FIXPK})."
  exit 92
fi


# Prevent same version going on
if [ "$FIXPK" = "$VERSION_FIXPK" ]; then
  echo "      (F) The TEDW Application:"
  echo "          \"$APP_DISPLAY_NAME\""
  echo "          with version ($APP_REL_VERSION) is already detected"
  echo "          on this system."
  echo "          To prevent destroying an existing installation,"
  echo "          you cannot reinstall.  You must explicitly"
  echo "          uninstall first."
  echo "          Exiting this application installation..."
  exit 93
# Prevent putting lower version
elif [ $FIXPK -lt $VERSION_FIXPK ]; then
  echo "      (F) You cannot install this product because the currently installed"
  echo "          versions is greater than the version you are"
  echo "          trying to install."
  echo "          Version you are trying to install = ${MAJOR}.${MINOR}.${MAINT}.{FIXPK}"
  exit 94
else
  echo "      (I) Fixpack version ($FIXPK) installable."
fi


echo "      (I) Application release version is acceptable and this patch can be installed."
return 0;
}


###


#
create_patch_dirs()
{
# create .installed dir
APP_INSTALLED_DIR="$APP_WORK_DIR/../.installed"
if [ ! -d "$APP_INSTALLED_DIR" ]; then
  blah=`mkdir -p "$APP_INSTALLED_DIR" 2>&1`
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to create application .installed directory"
    echo "          ($APP_INSTALLED_DIR)."
    exit 94
  fi

  # Set permissions
  blah=`chmod 755 "$APP_INSTALLED_DIR" `
fi


# Create an app log directory under install
APP_LOGDIR="$APP_WORK_DIR/install/logs/patch_$APP_REL_VERSION"
if [ ! -d "$APP_LOGDIR" ]; then
  blah=`mkdir -p "$APP_LOGDIR" 2>&1`
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to create application patch log directory at"
    echo "          ($APP_LOGDIR)."
    exit 94
  fi

  # Set permissions
  blah=`chmod -R 755 "$APP_WORK_DIR/install/logs" `
fi

return 0;
}


###


# Make a copy of the app's install subtree
create_app_dir_backup()
{
echo "==> Making backup of application subtree."
backup_marker="$APP_BACKUP_DIR/.tedw_${APP_PREV_VERSION}_backup.marker"


# Was this done for the patch?
if [ -r "$backup_marker" ]; then
  echo "      (I) Backup found for this version ($APP_PREV_VERSION)."
  echo "          Exiting this function..."
  return 0;
fi


# move the files
cp -rf "$APP_WORK_DIR" "$APP_BACKUP_DIR"
if [ $? -ne 0 ]; then 
  echo "      (F) Failed to make a backup copy of the Application media subtree."
  exit 95
fi

# Put a marker to know this was done correctly
touch "$backup_marker"


echo "      (I) Done backing up app subtree."
return 0;
}


###


# Strip files by the fixpack installed as to no rerun the logic
strip_files_for_delta()
{
dir_to_use="$1"
tmp_file="$TMPDIR/tmp_delta_filter.tmp"


#basic filter
if [ $APP_LATEST_FIXPACK_VER -eq 0 ]; then
  echo "          No Fixpacks detected on current install."
  echo "          No Delta work needed for ($dir_to_use)."
  echo "          Skipping..."
  echo ""
  return 0;
fi


ls "$dir_to_use" 2> /dev/null > "$TMPDIR/tmp_delta_list.dat"
file_count=`cat "$TMPDIR/tmp_delta_list.dat" | wc -l `
file_count=`echo $file_count | sed "s/        //g" ` # remove blanks


if [ "$file_count" != "0" ]; then
  echo "          Converting Delta files in ($dir_to_use)."
  echo "          Will filter Delta sections for fixpack $APP_LATEST_FIXPACK_VER and below:"

  while :
    do

    read file
    # Empty?
    if [ "$file" = "" ]; then
      break;
    fi

    echo "            Converting file:  ($file)"

    # Anything in it?
    blah=`cat "$dir_to_use"/$file | grep "\\-\\-#iftag" `
    if [ "$blah" = "" ]; then
      echo "                No tags found in file to remove.  Skipping..."
      echo ""
      continue;
    else
      echo "              * Found tags in file.  Removing non needed Delta sections..."
    fi

    # loop and Strip the versions not needed in the file for this fixpack
    loop_count=$APP_LATEST_FIXPACK_VER
    while [ $loop_count -gt 0 ];
      do

	# Strip the file
      cat "$dir_to_use"/$file | sed "/^--#iftag fixpack_$loop_count/,/--#endtag fixpack_$loop_count/d"  2> /dev/null  > "$tmp_file"
      mv -f "$tmp_file" "$dir_to_use"/$file

      echo "                  Filtering Fixpack ($loop_count) from file"
      loop_count=`expr $loop_count - 1 `
    done

    echo "                Done processing."
    echo ""
  done < "$TMPDIR"/tmp_delta_list.dat
  rm -f "$TMPDIR"/tmp_delta_list.dat
else
  echo "          No files in ($dir_to_use) directory to process as a Delta patch file."
fi


return 0;
}


###


# This function will be driven by the properties set in the
# twh_install_props.cfg file
move_media_to_prod_tree()
{
# CDW
if [ "$CDW_INCLUDED" = "TRUE" ]; then
  file_count1=`ls "$APP_MEDIA_DIR"/cdw/ddl  2> /dev/null  | wc -l `
  file_count2=`ls "$APP_MEDIA_DIR"/cdw/dml  2> /dev/null  | wc -l `

  file_count=`expr $file_count1 + $file_count2 `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No files found to install in CDW/DDL and CDW/DML directories."
  else
    echo "      (I) Moving files for CDW directory"  
    blah=`cp -rf "$APP_MEDIA_DIR"/cdw/ddl "$APP_WORK_DIR"/cdw `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory CDW/DDL at"
      echo "          ($APP_WORK_DIR)."
      exit 97
    fi

    blah=`cp -rf "$APP_MEDIA_DIR"/cdw/dml "$APP_WORK_DIR"/cdw `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory CDW/DML at"
      echo "          ($APP_WORK_DIR)."
      exit 97
    fi

    # Strip files to only include what needs to be installe for this fixpack (Delta support)
    echo "      (I) Running Delta work to filter files by fixpack for CDW."
    strip_files_for_delta "$APP_WORK_DIR/cdw/ddl";
    strip_files_for_delta "$APP_WORK_DIR/cdw/dml";
  fi

fi


# DOCS
if [ "$DOCS_INCLUDED" = "TRUE" ]; then
  file_count=`ls "$APP_MEDIA_DIR"/doc/  2> /dev/null  | wc -l `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No doc files found to install in Doc directory."
  else
    echo "      (I) Moving files for doc directory"
    blah=`cp -rf "$APP_MEDIA_DIR"/doc "$APP_WORK_DIR" `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory to doc"
      echo "          ($APP_WORK_DIR)."
      exit 95
    fi
  fi
fi


# ETL TAG
# Also we do not want versions on these files
if [ "$ETL_TAG_INCLUDED" = "TRUE" ]; then
  file_count=`ls "$APP_MEDIA_DIR"/etl/*.tag  2> /dev/null  | wc -l `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No ETL TAG files found to install in ETL directory."
  else
    echo "      (I) Moving ETL TAG file"
    # move the tag file
    blah=`rm "$APP_WORK_DIR"/etl/${AVA_CODE}*_dwc_data.tag `	# remove old

    blah=`cp -f "$APP_MEDIA_DIR"/etl/${AVA_CODE}*_dwc_data.tag  "$APP_WORK_DIR"/etl `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application tag file to ETL directory"
      exit 96
    fi
  fi
fi


# ETL SQL
if [ "$ETL_SQL_INCLUDED" = "TRUE" ]; then
  file_count=`ls "$APP_MEDIA_DIR"/etl/sql  2> /dev/null  | wc -l `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No ETL files found to install in ETL SQL directory."
  else
    echo "      (I) Moving files for ETL SQL directory"  
    blah=`cp -rf "$APP_MEDIA_DIR"/etl/sql "$APP_WORK_DIR"/etl `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory to ETL SQL"
      echo "          ($APP_WORK_DIR)."
      exit 97
    fi
  fi
fi


# i18n 
if [ "$I18N_INCLUDED" = "TRUE" ]; then
  file_count=`ls "$APP_MEDIA_DIR"/i18n/  2> /dev/null  | wc -l `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No i18n files found to install in i18n directory."
  else
    echo "      (I) Moving files for i18n directory"
    blah=`cp -rf "$APP_MEDIA_DIR"/i18n "$APP_WORK_DIR" `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory i18n"
      echo "          ($APP_WORK_DIR)."
      exit 98
    fi
  fi
fi


# Mart
if [ "$MART_INCLUDED" = "TRUE" ]; then
  file_count1=`ls "$APP_MEDIA_DIR"/mart/ddl  2> /dev/null  | wc -l `
  file_count2=`ls "$APP_MEDIA_DIR"/mart/dml  2> /dev/null  | wc -l `

  file_count=`expr $file_count1 + $file_count2 `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No Mart files found to install in Mart directory."
  else
    echo "      (I) Moving files for Mart directory"
    blah=`cp -rf "$APP_MEDIA_DIR"/mart/ddl "$APP_WORK_DIR"/mart `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory MART/DDL at"
      echo "          ($APP_WORK_DIR)."
      exit 97
    fi

    blah=`cp -rf "$APP_MEDIA_DIR"/mart/dml "$APP_WORK_DIR"/mart `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory MART/DML at"
      echo "          ($APP_WORK_DIR)."
      exit 97
    fi

    # Strip files to only include what needs to be installe for this fixpack (Delta support)
    echo "      (I) Running Delta work to filter files by fixpack for MART."
    strip_files_for_delta "$APP_WORK_DIR/mart/ddl";
    strip_files_for_delta "$APP_WORK_DIR/mart/dml";
  fi
fi


# Misc - Nomenclature up to the app teams (just replace)
file_count=`ls "$APP_MEDIA_DIR"/misc/  2> /dev/null  | wc -l `
file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
if [ "$file_count" = "0" ]; then
  echo "      (W) No Misc files found to install in Misc directory."
else
  echo "      (I) Moving files to misc directory"
  blah=`cp -rf "$APP_MEDIA_DIR"/misc "$APP_WORK_DIR" `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to copy application media to install directory to misc"
    echo "          ($APP_WORK_DIR)."
    exit 99
  fi
fi


# report - Copy with extensions
if [ "$REPORTS_INCLUDED" = "TRUE" ]; then
  file_count=`ls "$APP_MEDIA_DIR"/report/  2> /dev/null  | wc -l `
  file_count=`echo $file_count | sed "s/        //g" ` # remove blanks
  if [ "$file_count" = "0" ]; then
    echo "      (W) No Report files found to install in Report directory."
  else
    echo "      (I) Moving files for report directory"
    blah=`cp -rf "$APP_MEDIA_DIR"/report "$APP_WORK_DIR"/ `
    if [ $? -ne 0 ]; then
      echo "      (F) Unable to copy application media to install directory Report at"
      echo "          ($APP_WORK_DIR)."
      exit 97
    fi
  fi
fi


# Set permissions for all above directories
blah=`chmod -R 755 "$APP_WORK_DIR" `


return 0;
}


###


# Make a DB backup so they can manually recover on an error
execute_md_db_backup()
{
echo "==> Making backup of the database before installing Application information"


# Stop services first
echo "      (I) Stopping DB2 DWC NT services"
err_txt=`$DB2REGI 16 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to stop NT Service vwkernel and vwlogger"
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  exit 99
fi

# Stop DB2
echo "      (I) Stopping DB2"
err_txt=`$DB2CMD db2stop force 2>&1 `
if [ $? -ne 0 ]; then
  blah=`echo $err_txt | grep -i $sql_db_not_started `
  if [ $? -eq 0 ]; then
    :;			# If db not started that is OK
  else
    echo "      (F) Failed to stop DB2."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 100
  fi
fi

# Start DB2
echo "      (I) Starting DB2"
err_txt=`$DB2CMD db2start 2>&1 `
# Removed for defect 131629
#if [ $? -ne 0 ]; then
#  echo "      (F) Failed to start DB2."
#  echo "          Error is:"
#  echo "          $err_txt"
#  echo "          Exiting..."
#  exit 101
#fi


# Clean out old backups (keep one version)  Keep all versions
#echo "      (I) Cleaning up application backup directory"
#blah=`rm -rf "$APP_BACKUP_DIR"/* 2>&1 `
#if [ $? -ne 0 ]; then
#  echo "      (F) Failed trying to clean the ($APP_BACKUP_DIR) #directory"
#  exit 102
#fi


# Make the backup
echo "      (I) Making Backup"
blah=`$DB2EXEC_LOG -v backup db "$TWH_CTRL_DB" to \'$APP_BACKUP_DIR\' `
if [ $? -ne 0 ]; then
  echo "      (F) Failure detected during backup"
  echo "          for Database ($TWH_CTRL_DB)."
  echo "          Check messages in DB2 log ($DB2_INSTALL_LOG)."
  echo "          Exiting..."
  exit 103
fi


# Start services (logger has to be first!)
echo "      (I) Starting DB2 DWC NT services"
blah=`net start vwlogger `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0094E Failed to start NT Service vwlogger"
  echo "          Exiting..."
  clean_up
  exit 94
fi

blah=`net start vwkernel `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0095E Failed to start NT Service vwkernel"
  echo "          Exiting..."
  clean_up
  exit 95
fi


echo "      (I) Done making database backup."
return 0;
}


###


#
exec_cdw_updates()
{
ddl_drop_script="${AVA_CODE}_cdw_drop_schema.sql" 
ddl_script="${AVA_CODE}_cdw_schema.sql"
drop_triggers_script="${AVA_CODE}_cdw_drop_triggers.sql"
triggers_script="${AVA_CODE}_cdw_triggers.sql"
dml_script="${AVA_CODE}_cdw_data.sql"
dml_core_script="twh_cdw_data.sql"	 # TEDW Script
exec_dir="$APP_WORK_DIR/cdw"

echo "==> Updating CDW with delta based Application objects for patch"

# check the conn
gr_test_connection "$TWH_CDW_DB" "$COPT_CDW_DB2USER" "$COPT_CDW_DB2PASS";

# Drop Triggers
gr_build_script_name "$drop_triggers_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_CDW_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CDW_DB" $FALSE;

# Drop Schema
gr_build_script_name "$ddl_drop_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_CDW_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CDW_DB" $FALSE;

# Create Schema
gr_build_script_name "$ddl_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_CDW_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CDW_DB" $TRUE;

# Exec Triggers (We will use '#' as a delimiter for the triggers)
gr_build_script_name "$triggers_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_CDW_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_CDW_TYPE $CONN_TYPE_CONNECT "#";
gr_exec_sql_script "$TWH_CDW_DB" $TRUE "#";

### Data/DML time
echo "==> Updating CDW with Initial Application Reference Data"

# Insert TWH data (static script managed by TEDW team)
gr_build_script_name "$dml_core_script" "$exec_dir/dml";
gr_build_exec_script "$TWH_CDW_DB" "$exec_dir/dml" "$NAMMED_SCRIPT" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CDW_DB" $TRUE;

# Insert data
gr_build_script_name "$dml_script" "$exec_dir/dml";
gr_build_exec_script "$TWH_CDW_DB" "$exec_dir/dml" "$NAMMED_SCRIPT" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CDW_DB" $TRUE;


echo "     (I) Completed updating CDW for application"
return 0;
}


#
exec_mart_updates()
{
ddl_drop_script="${AVA_CODE}_mart_drop_schema.sql" 
ddl_script="${AVA_CODE}_mart_schema.sql"
drop_triggers_script="${AVA_CODE}_mart_drop_triggers.sql"
triggers_script="${AVA_CODE}_mart_triggers.sql"
dml_script="${AVA_CODE}_mart_data.sql"
exec_dir="$APP_WORK_DIR/mart"

echo "==> Updating Datamart DB with Application objects"

# check the conn
gr_test_connection "$TWH_DM_DB" "$COPT_MART_DB2USER" "$COPT_MART_DB2PASS";


# Drop Triggers
gr_build_script_name "$drop_triggers_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_DM_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_MART_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_DM_DB" $FALSE;

# Drop Schema
gr_build_script_name "$ddl_drop_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_DM_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_MART_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_DM_DB" $FALSE;

# Create Schema
gr_build_script_name "$ddl_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_DM_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_MART_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_DM_DB" $TRUE;

# Exec Triggers (We will use '#' as a delimiter for the triggers)
gr_build_script_name "$triggers_script" "$exec_dir/ddl";
gr_build_exec_script "$TWH_DM_DB" "$exec_dir/ddl" "$NAMMED_SCRIPT" $COPT_MART_TYPE $CONN_TYPE_CONNECT "#";
gr_exec_sql_script "$TWH_DM_DB" $TRUE "#";


### Data time
echo "==> Updating Datamart with Initial Application Reference Data"

# Insert data
gr_build_script_name "$dml_script" "$exec_dir/dml";
gr_build_exec_script "$TWH_DM_DB" "$exec_dir/dml" "$NAMMED_SCRIPT" $COPT_MART_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_DM_DB" $TRUE;


echo "      (I) Completed updating Datamart DB for application"
return 0;
}


###


#
exec_ctrl_updates()
{
rpi_script="twh_app_rpiStrings.sql"
exec_dir="$TWH_TOPDIR/install/sql"

echo "==> Updating RPI Metadata DB"

# check the conn
gr_test_connection "$TWH_CTRL_DB" "$COPT_CTRL_DB2USER" "$COPT_CTRL_DB2PASS";


gr_build_script_name "$rpi_script" "$exec_dir";
gr_build_exec_script "$TWH_CTRL_DB" "$exec_dir" "$NAMMED_SCRIPT" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;

echo "     (I) Completed updating RPI Metadata DB."

return 0;
}


###


# Build a custom ddl script that can run for each .csv file
# To call:      build_csv_report_script $1 $2 $3 ;
#       $1 = Database name
#       $2 = DDL/DML script file
#       $3 = csv data file
build_csv_report_script()
{
db_name="$1"
script_file="$2"
csv_file="$3"
create_table="$4"

exec_dir="$TWH_TOPDIR/rpi/sql"
csv_dir="$APP_WORK_DIR/report"
csv_script="$exec_dir/$script_file"
create_table_script="$exec_dir/$create_table"
TMP_DB_SCRIPT_FILE="$TMPDIR/tmp_$script_file"

conn_string="connect to $db_name user $COPT_CTRL_DB2USER using $COPT_CTRL_DB2PASS;"
import_string="import from '$csv_dir/$csv_file' of del insert into rpi.rpi_install_tmp;"
drop_table_string="drop table rpi.rpi_install_tmp;"

echo "      (I) Building report load script for ($script_file)"
# generic create table script
if [ ! -r "$create_table_script" ]; then
  echo "      (F) Could not locate ($create_table_script)."
  exit 104
fi

# csv script
if [ ! -r "$csv_script" ]; then
  echo "      (F) Could not locate ($csv_script) file."
  exit 105
fi

# csv data file
if [ ! -r "$csv_dir/$csv_file" ]; then
  echo "      (F) Could not locate ($csv_dir/$csv_file) file."
  exit 106
fi


# Now build the file  (gets a bit funky to put in blank lines)
echo "$conn_string" > "$TMP_DB_SCRIPT_FILE"
cat "$create_table_script" >> "$TMP_DB_SCRIPT_FILE"
echo "" >> "$TMP_DB_SCRIPT_FILE"
echo "$import_string"  >> "$TMP_DB_SCRIPT_FILE"
cat "$csv_script" >> "$TMP_DB_SCRIPT_FILE"
echo "" >> "$TMP_DB_SCRIPT_FILE"
echo "$drop_table_string" >> "$TMP_DB_SCRIPT_FILE"


if [ ! -r "$TMP_DB_SCRIPT_FILE" ]; then
  echo "      (F) Error trying to read Created DDL execution script"
  rm -f "$TMP_DB_SCRIPT_FILE"
  exit 107
elif [ -x "$TMP_DB_SCRIPT_FILE" ]; then
  chmod 777 "$TMP_DB_SCRIPT_FILE"
fi


echo "      (I) Done building report loading script"
return 0;
}


###


# Load data into RPI for predefined reports (CSV file processing)
install_reports()
{
ct1="rpi_export1_create_table.sql"
ct2="rpi_export2_create_table.sql"
ct3="rpi_export3_create_table.sql"
ct4="rpi_export4_create_table.sql"
sql1="rpi_export1_process.sql"
sql2="rpi_export2_process.sql"
sql3="rpi_export3_process.sql"
sql4="rpi_export4_process.sql"
# files supplied by app teams
csv1="${AVA_CODE}_rpi_export1.csv"
csv2="${AVA_CODE}_rpi_export2.csv"
csv3="${AVA_CODE}_rpi_export3.csv"
csv4="${AVA_CODE}_rpi_export4.csv"


sql_drop_table="drop table rpi.rpi_install_tmp;"

sql_string="
UPDATE RPI.DATAMART
   SET DM_ALIAS = '$TWH_DM_DB',
       DM_USER = '$COPT_MART_DB2USER',
       DM_SCHEMA = '$APP_DATAMART_SCHEMA',
       DM_URL = 'jbdc:db2:$TWH_DM_DB',
       DM_PWD_ENCRYPT = '$COPT_MART_DB2PASS'
   where DM_ALIAS is NULL
     and DM_USER is NULL
     and DM_SCHEMA is NULL
     and DM_URL is NULL
     and DM_PWD_ENCRYPT is NULL
;"

echo "==> Installing pre-packaged RPI reports"

# Skip this if directed to do so
if [ "$REPORTS_INCLUDED" != "TRUE" ]; then
  echo "      (I) Skipping Report installation as requested."
  return 0;
fi 


# Clean up the tmp table - Must Ignore errors here
echo "      (I) Cleaning up CSV processing tmp table"
echo "$sql_drop_table" > "$TMPDIR"/twh_runtime_drop_tables.sql
gr_build_exec_script "$TWH_CTRL_DB" "$TMPDIR" "twh_runtime_drop_tables.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;
rm -f "$TMPDIR/twh_runtime_drop_tables.sql"


# We need to build each script for each parallel .csv file
build_csv_report_script "$TWH_CTRL_DB" "$sql1" "$csv1" "$ct1";
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

build_csv_report_script "$TWH_CTRL_DB" "$sql2" "$csv2" "$ct2";
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

build_csv_report_script "$TWH_CTRL_DB" "$sql3" "$csv3" "$ct3";
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

build_csv_report_script "$TWH_CTRL_DB" "$sql4" "$csv4" "$ct4";
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# Update the RPI DB with connect values for the reports.
echo "$sql_string" > "$TMPDIR"/twh_runtime_conn_string.sql
gr_build_exec_script "$TWH_CTRL_DB" "$TMPDIR" "twh_runtime_conn_string.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;
rm -f "$TMPDIR/twh_runtime_conn_string.sql"

echo "      (I) Completed installed RPI reports"
return 0;
}


###


# Look for all exported files
exec_app_export_health()
{
export_dir1="$APP_WORK_DIR/etl/export"
file_list="AGENTSITE.del
ATTRIBUTE.del
ATTRIBUTELINK.del
BINARYDATA.del
BUSINESSVIEW.del
BVALIAS.del
BVSCHEDULE.del
CALDATE.del
CONFIGURATION.del
CONVERSIONATTR.del
CORRELATION.del
CUSTCAL.del
DATARESOURCE.del
EDITION.del
EDITIONREL.del
FINDATTRDB2.del
INFORESOURCE.del
IWHSTRINGSlong.del
IWHSTRINGSshort.del
KEY.del
KEYMAP.del
LOGMSGS.del
MAP.del
OLAPCUBE.del
PROCESS.del
PROGFUNC2.del
PROGPARM.del
PROGTYPE.del
RELATIONSHIP.del
SECURITYGROUP.del
STARSCHEMA.del
STATISTICS.del
SUBJECT.del
UDPGROUP.del
USERFUNCTION.del
VWUSER.del
"

echo "      (I) Checking that all App DB export files exist"

# Spin through the list and ensure the file is there (36 files)
for file_to_find in $file_list
  do
  if [ ! -r "$export_dir1/$file_to_find" ]; then
    echo "      (W) Export file ($file_to_find) missing."
    return $FALSE;
  fi

done


echo "      (I) Done checking for export files"
return $TRUE;
}


###


# Clean out existing values in MD DB for this app
drop_dwc_data_4_update()
{
etl_sql_dir="$TWH_TOPDIR/install/sql"
sql_error1="SQL0204N"
sql_db_not_started="SQL1032N"
dwc_report="$TMPDIR/${AVA_CODE}_dwc_upgrade_relationships.report"


echo ""
echo "==> Removing the application design (ETL) from the Metadata database."
echo ""

# If they do not have a backup, skip this section
if [ ! -r "$TWH_TOPDIR/apps_backups/$AVA_CODE/install/$TWH_CTRL_DB.0"  ]; then
  echo "      (W) No DB backup found.  Must not have run metadata DB"
  echo "          process steps during this application's install."
  echo "          Continuing..."
  echo ""

  return 0;
fi


# Check to see if they made it to the export on the install of this App
exec_app_export_health;
have_all=$?

# If they have a backup and no export, tell them they need to restore
if [ $have_all -eq $FALSE ]; then
  echo ""
  echo ""
  echo ""
  echo ""
  echo ""
  echo ""
  echo "      (W) Could not locate all application export files to restore"
  echo "          the ($TWH_CTRL_DB) to the state it was in before this"
  echo "          application installation."
  echo ""
  echo "          You need to manually restore the database back to"
  echo "          it's state before the last application installation."
  echo ""
  echo "          To recover to the state prior to this application"
  echo "          installation:"
  echo ""
  echo "          1).  Allow this deinstall to finish after this pause"
  echo ""
  echo "          2).  Restore the DWC MD database manually by doing"
  echo "               the following:"
  echo ""
  echo "            db2 -v restore db $TWH_CTRL_DB from"
  echo "            $TWH_TOPDIR/apps_backups/$AVA_CODE/install/$TWH_CTRL_DB.0"
  echo ""
  echo "          3).  Before reinstalling, determine the reason"
  echo "               why the application failed and resolve it."
  echo ""
  echo "          <Pausing for 60 seconds to allow you to read this warning>"
  echo ""
  echo ""
  echo ""
  echo ""
  echo ""
  echo ""

  sleep 60
  echo "          Continuing with deinstall..."

  return 0;		# exit this routine
fi


# Stop services first
echo "      (I) Stopping DB2 DWC NT services"
err_txt=`$DB2REGI 16 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0086E Failed to stop NT Service vwkernel and vwlogger"
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  clean_up
  exit 86
fi

# Stop DB2
echo "      (I) Stopping DB2"
err_txt=`$DB2CMD db2stop force 2>&1 `
if [ $? -ne 0 ]; then
  blah=`echo $err_txt | grep -i $sql_db_not_started `
  if [ $? -eq 0 ]; then
    :;			# If db not started that is OK
  else
    echo "      (F) CDWIA0087E Failed to stop DB2."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    clean_up
    exit 87
  fi
fi

# Start DB2
echo "      (I) Starting DB2"
err_txt=`$DB2CMD db2start 2>&1 `
# Removed for defect 131629
#if [ $? -ne 0 ]; then
#  echo "      (F) CDWIA0088E Failed to start DB2."
#  echo "          Error is:"
#  echo "          $err_txt"
#  echo "          Exiting..."
#  clean_up
#  exit 88
#fi

# Clean out old backups  (keep only one)
#echo "      (I) Cleaning up application backup directory"
#blah=`rm -rf "$APP_BACKUP_DIR"/* 2>&1 `
#if [ $? -ne 0 ]; then
#  echo "      (F) CDWIA0089E Failed trying to clean the 3($APP_BACKUP_DIR) directory"
#  clean_up
#  exit 89
#fi

# Make the backup
echo "      (I) Making Backup"
blah=`$DB2EXEC_LOG -v backup db "$TWH_CTRL_DB" to \'$APP_BACKUP_DIR\' `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0090E Failure detected during backup"
  echo "          for Database ($TWH_CTRL_DB)."
  echo "          Check messages in DB2 log ($DB2_INSTALL_LOG)."
  echo "          Exiting..."
  TWH_KEEP_TMP_FILES_TRUE=1
  clean_up
  exit 90
fi

# Test a connection
echo "      (I) Testing the DB connection"
err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$DB2USER" using "$DB2PASS" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0091E Unable to connect to database ($TWH_CTRL_DB)."
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  clean_up
  exit 91
fi

# Have to do this again in case db2bp thread was forced
err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$COPT_CTRL_DB2USER" using "$COPT_CTRL_DB2PASS" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0091E Unable to connect to database ($TWH_CTRL_DB)."
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  clean_up
  exit 91
fi

# Check for IWH not in known list (script requires 35 tables)
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "iwhcheck1.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;

# Check for IWH not in known list
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "iwhcheck2.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;

# Drop the delta tables
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "drop_delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;

# Create the Delta table
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "create_delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Import the delta tables from the export
# Move to the correct dir to support
# relative import path
echo "      (I) Starting the import to delete the App data from the"
echo "          $TWH_CTRL_DB.  If this fails:"
echo ""
echo "          A backup of the database will be taken before this"
echo "          action is executed."
echo "          To restore data, execute the following DB2 command:"
echo ""
echo "            db2 -v restore db $TWH_CTRL_DB from"
echo "            $TWH_TOPDIR/apps_backups/$AVA_CODE/deinstall/$TWH_CTRL_DB.0"
echo ""

cur_dir=`pwd`
cd "$APP_WORK_DIR/etl"
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "import.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;
cd "$cur_dir"


# Drop tables for the clean script
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "clean_drop_tables.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Run the list_obj to dump the custom relationships the customer did for this app
rm -f "$dwc_report"

gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "list_obj.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
echo "    **(I) Running report to collect DWC customizations."
$DB2EXEC_LOG_DDL \"$TMP_DB_SCRIPT_FILE\" 2> /dev/null > "$dwc_report"

echo "          Data Warehouse Center customizations report located at ($dwc_report)."
rm -f "$TMP_DB_SCRIPT_FILE" 


# Run the clean script
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "clean.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Drop tables after the clean script
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "clean_drop_tables.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# DO THE REMOVE!!!
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "remove.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Drop the delta tables
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "drop_delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Start services (logger has to be first!)
echo "      (I) Starting DB2 DWC NT services"
blah=`net start vwlogger `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0094E Failed to start NT Service vwlogger"
  echo "          Exiting..."
  clean_up
  exit 94
fi

blah=`net start vwkernel `
if [ $? -ne 0 ]; then
  echo "      (F) CDWIA0095E Failed to start NT Service vwkernel"
  echo "          Exiting..."
  clean_up
  exit 95
fi


echo "      (I) Completed removing the Metadata database objects"
return 0
}


###


# Tag file executed into the DWC Metadata DB
import_etl()
{
etl_sql_dir="$TWH_TOPDIR/install/sql"
sql_error1="SQL0204N"
sql_db_not_started="SQL1032N"


echo "==> Importing the application design (ETL) into CDW Metadata schema"
echo "      A backup of the database will be taken before this"
echo "      action is executed."
echo "      To restore data, execute the following DB2 command:"
echo ""
echo "      db2 -v restore db $TWH_CTRL_DB from "
echo "      $TWH_TOPDIR/apps_backups/$AVA_CODE/install/$TWH_CTRL_DB.0"
echo ""


# Stop services first
echo "      (I) Stopping DB2 DWC NT services"
err_txt=`$DB2REGI 16 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to stop NT Service vwkernel and vwlogger"
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  exit 108
fi

# Stop DB2
echo "      (I) Stopping DB2"
err_txt=`$DB2CMD db2stop force 2>&1 `
if [ $? -ne 0 ]; then
  blah=`echo $err_txt | grep -i $sql_db_not_started `
  if [ $? -eq 0 ]; then
    :;			# If db not started that is OK
  else
    echo "      (F) Failed to stop DB2."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 109
  fi
fi

# Start DB2
echo "      (I) Starting DB2"
err_txt=`$DB2CMD db2start 2>&1 `
# Removed for defect 131629
#if [ $? -ne 0 ]; then
#  echo "      (F) Failed to start DB2."
#  echo "          Error is:"
#  echo "          $err_txt"
#  echo "          Exiting..."
#  exit 110
#fi

# Test a connection
echo "      (I) Testing the DB connection"
err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$COPT_CTRL_DB2USER" using "$COPT_CTRL_DB2PASS" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Unable to connect to database ($TWH_CTRL_DB)."
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  exit 111
fi

# Have to do this again in case db2bp thread was forced
err_txt=`$DB2EXEC_LOG connect to "$TWH_CTRL_DB" user "$COPT_CTRL_DB2USER" using "$COPT_CTRL_DB2PASS" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Unable to connect to database ($TWH_CTRL_DB)."
  echo "          Error is:"
  echo "          $err_txt"
  echo "          Exiting..."
  exit 112
fi

# Check for IWH not in known list (script requires 35 tables)
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "iwhcheck1.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# Check for IWH not in known list
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "iwhcheck2.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# Drop the before tables  
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "drop_before.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;

# Drop the delta tables
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "drop_delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $FALSE;


# Create the before table
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "create_before.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;


# Create the Delta table
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "create_delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# Backup metadata to the Before tables
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "before.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;


# WH Import the metadata  (There may be more than one .tag file)
echo "      (I) Importing Data into the DB2 DWC."

# Handle multiple tag files
ls "$APP_WORK_DIR/etl"/*_dwc_data.tag > "$TMPDIR/tagfiles";

while :
do
  read tag_file

  if [ "$tag_file" = "" ]; then break; fi

  echo "      (I) Importing from ($tag_file)"
  err_txt=`$DB2IMP \"$tag_file\" \"$TMPDIR\" $TWH_CTRL_DB $COPT_CTRL_DB2USER $COPT_CTRL_DB2PASS PREFIX=IWH 2>&1 `
  rc=$?
  # Trap errors and warnings
  if [ $rc -ge 3 ]; then
    echo "      (F) Failed to import the metadata into DB2 DWC"
    echo "          Error is:"
    echo "          $err_txt"
    echo "          To recover to the state prior to this application"
    echo "          installation"
    echo ""
    echo "          1).  Run the application deinstaller for this App"
    echo "          2).  Restore the DWC MD database manually as"
    echo "               defined above"
    echo "          3).  Before reinstalling, determine the reason"
    echo "               why the application failed and resolve"
    echo "               before reinstalling."
    echo ""
    echo "          Exiting..."
    exit 113
  elif [ $rc -gt 0 -a $rc -lt 3 ]; then
    echo "      (W) Warning detected during import"
    echo "          Continuing"

  fi

# Move the tag file log
log_name=`echo "$tag_file" | sed "s/\.tag/\.log/" `
log_name=`basename "$log_name" `
mv "$TMPDIR"/$log_name  "$APP_LOGDIR"

done < "$TMPDIR/tagfiles"

# delete tag file list
rm -f "$TMPDIR/tagfiles"
echo "      (I) ETL data imported successfully"


# Get the Delta
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

#Empty the export directory
echo "      (I) Cleaning Export directory"
blah=`rm -rf "$APP_EXPORT_DIR"/* 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Failed trying to clean the ($APP_EXPORT_DIR) directory"
  exit 114
fi


# Export the CSV and lob files
echo "      (I) Changing working directory to export data"
cur_dir=`pwd`
cd "$APP_WORK_DIR/etl"

gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "export.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# go back
echo "      (I) Changing back to original working directory"
cd "$cur_dir"


# Drop Before tables
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "drop_before.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;

# Drop the delta tables
gr_build_exec_script "$TWH_CTRL_DB" "$etl_sql_dir" "drop_delta.sql" $COPT_CTRL_TYPE $CONN_TYPE_CONNECT;
gr_exec_sql_script "$TWH_CTRL_DB" $TRUE;


# Start services (logger has to be first!)
echo "      (I) Starting DB2 DWC NT services"
blah=`net start vwlogger `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to start NT Service vwlogger"
  echo "          Exiting..."
  exit 115
fi

blah=`net start vwkernel `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to start NT Service vwkernel"
  echo "          Exiting..."
  exit 116
fi


echo "      (I) Completed importing ETL into the database"
return 0;
}


###


# This utility will load the RPI reporting translated values into
# the db for use by other applications
# We expect InstallShield will prepare all the necessary .jar files
# include the jdbc.zip in the top/install/bin
# This should only run on NT
#
# Patch note - The files should include original defs and the java prog
# will overwrite the values with same vals
load_app_nls_into_db()
{
JAVA="$TWH_TOPDIR/_jvm/bin/java"
JAVA_DIR="$TWH_TOPDIR/_jvm"
TWH_JAR_DIR="$TWH_TOPDIR/tools/bin"
APP_BUNDLES_DIR="$APP_WORK_DIR/i18n"
PROP_FILE="$TMPDIR/twh_nls_props.cfg"
lang_list="en, fr, de, es, it, ja, ko, pt_BR, zh_CN, zh_TW"
clean_db_name=`echo $TWH_CDW_DB | sed "s/        //g" `


# Upper the class name only
AVA_UPPR=`echo $AVA_CODE | tr '[a-z]' '[A-Z]' `

# *** Start of file output
write_string="# NLS Application DB Strings loader Conf File
#
JDBC.driver=COM.ibm.db2.jdbc.app.DB2Driver
JDBC.url=jdbc:db2:${clean_db_name}
JDBC.id=${COPT_CDW_DB2USER}
JDBC.password=${COPT_CDW_DB2PASS}
#
1.rb.class=com.ibm.twh.${AVA_CODE}.nls.${AVA_UPPR}nls
1.rb.locale=${lang_list}
"
# *** End of file output


# Write NLS config file
echo "      (I) Writing NLS input config file"
echo "$write_string" > "$PROP_FILE"


# Prepare a class path
TWH_CLASSPATH="$TWH_JAR_DIR/com.tivoli.twh.nls@1.1.0.jar;$TWH_JAR_DIR/log.jar;$APP_BUNDLES_DIR;$TWH_JAR_DIR/db2java.zip;$APP_BUNDLES_DIR/com.ibm.twh.$AVA_CODE.nls@$APP_REL_VERSION.jar;$APP_BUNDLES_DIR/com.ibm.twh.$AVA_CODE.nls_intl@$APP_REL_VERSION.jar"


echo "==> Invoking the NLS Database import utility."

# Invoke the Java executable to import the resource bundles
txt=`"$JAVA" -cp "$TWH_CLASSPATH" com.tivoli.twh.nls.TWHImportBundles "$PROP_FILE" 2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Failed to execute NLS Java DB2 import."
  echo "          Error text is:"
  echo "          $txt"
  echo "          Exiting..."
  exit 117
fi

# dump warnings to screen
echo "      (I) NLS import messages were:"
echo "$txt"
echo ""

# Remove the props file.  Has the db pass in it.
echo "      (I) Removing NLS input config file"
rm -f "$PROP_FILE"

echo "      (I) Completed load of application NLS strings into Database"
return 0;
}


###


# Start and stops PS servers
manage_ps_servers()
{
ps_action=$1


# Env setup
if [ $UNIX_INTERP -eq $FALSE ]; then
  ps_exec_path="$PS_HOME/bin/w32-ix86"
  ps_exec_path_private="$PS_HOME/bin/private/w32-ix86"
  extension=".bat"
  generic_jar_path=`echo "${PATCH_GENERIC_DIR}/ps/" | sed 's/\//\\\\/g'`
else
  echo "            Path used will be 'generic_unix'"
  ps_exec_path="$PS_HOME/bin/generic_unix"
  ps_exec_path_private="$PS_HOME/bin/private/generic_unix"
  extension=".sh"
  wcmd_prog="$ps_exec_path_private/wcmd.sh"
  generic_jar_path="${PATCH_GENERIC_DIR}/ps/"
fi


if [ ! -d "$PS_HOME" ]; then 
  echo "      (F) Could not locate PS home directory ($PS_HOME)"
  exit 132
fi


# Execute requested action
if [ $ps_action -eq $STOP_PS -o $ps_action -eq $RECYCLE_PS ]; then
  # Stop servers
  echo "          Stopping MCR and WC servers"
  blah=`"$ps_exec_path_private"/stopwc${extension} 2>&1 `
  blah=`"$ps_exec_path_private"/stopmcr${extension} 2>&1 `
fi
  

if [ $ps_action -eq $START_PS -o $ps_action -eq $RECYCLE_PS ]; then
  echo "          Starting MCR server"
  blah=`"$ps_exec_path"/mcr${extension} 2>&1 `
  
  # Wait for them to start...
  echo "          Waiting for MCR server to start"
  blah=`"$ps_exec_path_private"/waitmcr${extension} 2>&1 `
  rc=$?
  if [ $rc -ne 0 ]; then
    echo "      (F) Failed to run waitmcr program.  RC=($rc)"
    echo "          Error was:"
    echo "          $blah"
    exit 133
  fi
  
  echo "          Starting WC server"
  blah=`"$ps_exec_path"/wc${extension} 2>&1 `
fi


return 0;
}


###


# Install the message files for RPI
install_lang_files()
{
echo "==> Installing RPI language files"
ps_inst_dir="$PS_HOME/cd"



echo "      (I) Setting up exec path and run variables for PS"
# Env setup
if [ $UNIX_INTERP -eq $FALSE ]; then
  echo "            Path used will be 'w32-ix86'"
  generic_jar_path=`echo "${PATCH_GENERIC_DIR}/ps/" | sed 's/\//\\\\/g'`
 

  # PS workaround for wcmd.bat bug
  wcmd_prog="_wcmd.bat"
  ps_win_path=`echo "$PS_HOME/bin/private/w32-ix86" | sed 's/\//\\\\/g'`
  wcmd_fix_file="@echo off
REM Runtime TEDW fix for PS wcmd.bat problem

call ${ps_win_path}\wcmd.bat "%*"
exit %ERRORLEVEL%
"	# End file content

  # Write tmp bat file fix
  echo "$wcmd_fix_file" > "$TMPDIR"/$wcmd_prog
  chmod 777 "$TMPDIR"/$wcmd_prog
else
  echo "            Path used will be 'generic_unix'"
  wcmd_prog="$ps_exec_path_private/wcmd.sh"
  generic_jar_path="${PATCH_GENERIC_DIR}/ps/"
fi


# Start server incase...
#manage_ps_servers $START_PS;


if [ ! -w "$PS_HOME"/cd ]; then
  echo "      (F) Cannot write to ($PS_HOME/cd) directory."
  exit 116
fi


file_count=`ls "$APP_MEDIA_DIR"/i18n/ | wc -l `
if [ "$file_count" = "0" ]; then
  echo "      (W) No language files found to install for RPI."
  rm -f "$TMPDIR"/$wcmd_prog
  return 0;
fi

# Check the PS directory
if [ ! -d "$ps_inst_dir" -o ! -w "$ps_inst_dir" ]; then
  echo "      (F) Cannot find/access the PS home directory"
  echo "          Directory supplied is ($ps_inst_dir)."
  exit 118
fi

# Now move all the application media into PS
echo "      (I) Moving files into PS directory"
blah=`cp -rf "$APP_WORK_DIR/i18n"/* "$ps_inst_dir"  2>&1 `
if [ $? -ne 0 ]; then
  echo "      (F) Unable to copy the application message files"
  echo "          into PS ($ps_inst_dir) directory."
  exit 119
fi

echo "      (I) Stopping MCR and WC servers to Recycle servers"
manage_ps_servers $RECYCLE_PS;


# Cleanup
if [ $UNIX_INTERP -eq $FALSE ]; then
  rm -f "$TMPDIR"/$wcmd_prog
fi


echo "      (I) Complete installing language files"
return 0;
}


###


# Add dirs not included in 1.1 for app installer to synch all component 
# hosts
build_rpi_dirs()
{
echo "==> Rebuild Application tree for RPI host to synch with other hosts"


if [ ! -d "$APP_WORK_DIR/cdw/ddl" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/cdw/ddl" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/cdw/ddl)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/cdw/dml" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/cdw/dml" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/cdw/dml)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/doc" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/doc" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/doc)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/etl/sql" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/etl/sql" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/etl/sql)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/i18n" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/i18n" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/i18n)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/mart/ddl" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/mart/ddl" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/mart/ddl)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/mart/dml" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/mart/dml" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/mart/dml)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/misc" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/misc" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/misc)"
  fi
fi

if [ ! -d "$APP_WORK_DIR/report" ]; then
  blah=`mkdir -p "$APP_WORK_DIR/report" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Could not create directory ($APP_WORK_DIR/report)"
  fi
fi


# Set permissions for all above directories
blah=`chmod -R 755 "$APP_WORK_DIR" `


echo "==> Done rebuilding Application tree for RPI host to synch with other hosts"
return 0;
}


###


# If the app team has a pre scirpt, kick it off
# We pass to parms into the app pre-script
#	$1 = The TWH config file
#	$2 = the app working directory
exec_pre_script()
{
echo "==> Preparing to Execute the Application Pre Script"

# If they specify 'null' as the prop value, skip the pre-script step
prescript_null=`echo "$PRE_SCRIPT" | tr '[a-z]' '[A-Z]' `
if [ "$prescript_null" = "NULL" ]; then
  echo "      (I) No application Pre script specified for this appliction."
  return 0;
fi

# Set file and path
pre_script_file="$APP_WORK_DIR/$PRE_SCRIPT"

# is the script really here
if [ ! -r "$pre_script_file" ]; then
  echo "      (F) Error trying to read/execute application Pre Script."
  echo "          File is ($pre_script_file)."
  exit 121
elif [ ! -x "$pre_script_file" ]; then
  chmod 777 "$pre_script_file"
fi

# Build a log file and fire off the Pre script
log_file=`basename "$pre_script_file"`
log_file_name="${log_file}.log"
tmp_log_file_name="tmp_${log_file}.log"

echo "      (I) Executing application Pre script"
echo "          Application Execution log is ($TMPDIR/$tmp_log_file_name)"
"$pre_script_file" "$TWH_CONFIG_FILE" "$APP_WORK_DIR" > "$TMPDIR/$tmp_log_file_name" 2>&1
if [ $? -ne 0 ]; then
  echo "      (F) Failure during run of application Pre script."
  echo "          Check the Application Execution log for details"
  exit 255
fi

# move log to app logs dir
mv "$TMPDIR"/$tmp_log_file_name "$APP_LOGDIR"/$log_file_name


echo "      (I) Completed preparing and executing the application Pre script"
return 0;
}


###


# If the app team has a Pre ETL scirpt, kick it off
# We pass to parms into the app pre etl script
#	$1 = The TWH config file
#	$2 = the app working directory
exec_pre_etl_script()
{
echo "==> Preparing to Execute the Application PRE ETL Script"

# If they specify 'null' as the prop value, skip the pre etl script 
preetlscript_null=`echo "$PRE_ETL_SCRIPT" | tr '[a-z]' '[A-Z]' `
if [ "$preetlscript_null" = "NULL" ]; then
  echo "      (I) No application Pre ETL script specified for this appliction."
  return 0;
fi

# Set file and path
preetl_script_file="$APP_WORK_DIR/$PRE_ETL_SCRIPT"

# is the script really here
if [ ! -r "$preetl_script_file" ]; then
  echo "      (F) Error trying to read/execute application Pre ETL Script."
  echo "          File is ($preetl_script_file)."
  exit 124
elif [ ! -x "$preetl_script_file" ]; then
  chmod 777 "$preetl_script_file"
fi

# Build a log file and fire off the pre etl script
log_file=`basename "$preetl_script_file"`
log_file_name="${log_file}.log"
tmp_log_file_name="tmp_${log_file}.log"

echo "      (I) Executing application Pre ETL script"
echo "          Application Execution log is ($TMPDIR/$tmp_log_file_name)"
"$preetl_script_file" "$TWH_CONFIG_FILE" "$APP_WORK_DIR" > "$TMPDIR/$tmp_log_file_name" 2>&1
if [ $? -ne 0 ]; then
  echo "      (F) Failure during run of application Pre ETL script."
  echo "          Check the Application Execution log for details"
  exit 255
fi

# move log to app logs dir
mv "$TMPDIR"/$tmp_log_file_name "$APP_LOGDIR"/$log_file_name


echo "      (I) Completed preparing and executing the application Pre ETL script"
return 0;
}


###


#
test_parent_wpack_prereq()
{
loop_count=0

echo "==> Checking if another Warehouse Pack is required before"
echo "    this one can be instaled."


# Determine which if prop has value and execute on it (up to 5 dependencies)
while :
  do
  loop_count=`expr $loop_count + 1 `

  case "$loop_count" in

      #
      1)
	prop_name="$WPACK_PREREQ_AVA_NAME_1"
	prop_ver="$WPACK_PREREQ_VERSION_1"
        ;;

      #
      2)
	prop_name="$WPACK_PREREQ_AVA_NAME_2"
	prop_ver="$WPACK_PREREQ_VERSION_2"
        ;;

      #
      3)
	prop_name="$WPACK_PREREQ_AVA_NAME_3"
	prop_ver="$WPACK_PREREQ_VERSION_3"
        ;;

      #
      4)
	prop_name="$WPACK_PREREQ_AVA_NAME_4"
	prop_ver="$WPACK_PREREQ_VERSION_4"
        ;;

      #
      5)
	prop_name="$WPACK_PREREQ_AVA_NAME_5"
	prop_ver="$WPACK_PREREQ_VERSION_5"
        ;;

      #
      6)
	break;		# Exit as there is only 5 rules allowed
        ;;


      # Default - is an error
      *)
        echo "          Illegal value.  Count exceeded 5 ($loop_count)"
        exit 1
        ;;
    esac


  # Loop in order.  If they specify 'null' as the prop value, exit further processing
  wpack_prereq_null=`echo "$prop_name" | tr '[a-z]' '[A-Z]' `
  if [ "$wpack_prereq_null" = "NULL" ]; then
    if [ $loop_count -eq 1 ]; then
      echo "          No requirements found."
      echo "          Exiting Wpack Prereq loop."
      return 0;
    else
      echo "          Done processing for pre requirements."
      echo "          Exiting Wpack Prereq loop."
      return 0;
    fi
  fi

  # Ensure it is in lower case
  prop_name=`echo "$prop_name" | tr '[A-Z]' '[a-z]' `
  echo "      (I) Checking Pre-required Warehouse Pack rule for ($prop_name)."


  # Must have a requirement to check - now pull version
  wpack_prereq_null=`echo "$prop_ver" | tr '[a-z]' '[A-Z]' `
  if [ "$wpack_prereq_null" = "NULL" ]; then
    echo "      (F) The value for property WPACK_PREREQ_VERSION_${loop_count} cannot"
    echo "          be null when you have specified a required Warehouse Pack."
    exit 150;
  fi


  # Now validate the app is on
  if [ ! -r "$TWH_TOPDIR"/apps/${prop_name} ]; then
    echo "      (F) Pre-required application not detected on this system."
    echo "          You cannot install this patch without first"
    echo "          installing the Warehoues Pack ($prop_name)"
    echo "          version ($prop_ver)"
    echo "          Exiting..."
    exit 151
  else
    echo "      (I) Pre-required Warehouse Pack detected."
  fi


  breakdown_version_levels "$prop_ver";
  check_version "$prop_name" "$prop_ver";

  # Test for minimum version level
  if [ $MAJOR -le $VERSION_MAJOR -a $MINOR -le $VERSION_MINOR -a $MAINT -le $VERSION_MAINT -a $FIXPK -le $VERSION_FIXPK ]; then
    echo "      (I) Pre-required Warehouse Pack version ($prop_ver) acceptable."
  else
    echo "      (F) Pre-required version not detected on this system."
    echo "          You cannot install this patch without first"
    echo "          installing ($prop_name)"
    echo "          at version ($prop_ver)"
    echo "          Exiting..."
    exit 152
  fi


  echo ""
done


echo "      (I) This application patch process can proceed."
echo "          Done validating WPack pre-req version"
return 0;
}


###


# If the app team has a after scirpt, kick it off
# We pass to parms into the app afterscript
#	$1 = The TWH config file
#	$2 = the app working directory
exec_after_script()
{
echo "==> Preparing to Execute the Application After Script"

# If they specify 'null' as the prop value, skip the afterscript step
afterscript_null=`echo "$AFTER_SCRIPT" | tr '[a-z]' '[A-Z]' `
if [ "$afterscript_null" = "NULL" ]; then
  echo "      (I) No application after script specified for this appliction."
  return 0;
fi

# Set file and path
after_script_file="$APP_WORK_DIR/$AFTER_SCRIPT"

# is the script really here
if [ ! -r "$after_script_file" ]; then
  echo "      (F) Error trying to read/execute application After Script."
  echo "          File is ($after_script_file)."
  exit 129
elif [ ! -x "$after_script_file" ]; then
  chmod 777 "$after_script_file"
fi

# Build a log file and fire off the after script
log_file=`basename "$after_script_file"`
log_file_name="${log_file}.log"
tmp_log_file_name="tmp_${log_file}.log"

echo "      (I) Executing application after script"
echo "          Application Execution log is ($TMPDIR/$tmp_log_file_name)"
"$after_script_file" "$TWH_CONFIG_FILE" "$APP_WORK_DIR" > "$TMPDIR/$tmp_log_file_name" 2>&1
if [ $? -ne 0 ]; then
  echo "      (F) Failure during run of application after script."
  echo "          Check the Application Execution log for details"
  exit 255
fi

# move log to app logs dir
mv "$TMPDIR"/$tmp_log_file_name "$APP_LOGDIR"/$log_file_name


echo "      (I) Completed preparing and executing the application after script"
return 0;
}


###


# If the app team has a after scirpt, kick it off
exec_core_prescript()
{
echo "==> Preparing to Execute the Core Pre Script"

pre_script_file="$TWH_TOPDIR/install/bin/twh_core_pre.sh"
tmp_log_file="tmp_core_prescript.log"


if [ ! -r "$pre_script_file" ]; then
  echo "      (I) No Core prescript found."
  echo "          Exiting function..."
  return 0;
fi


if [ ! -x "$pre_script_file" ]; then
  chmod 777 "$pre_script_file"
fi

# Fake a couple of env vars for this to run
APP_INSTALL_LIST_FILE="blah"; export APP_INSTALL_LIST_FILE


echo "      (I) Executing Core Pre script"
echo "          Core Execution log is ($TMPDIR/$tmp_log_file)"
"$pre_script_file" > "$TMPDIR/$tmp_log_file" 2>&1
if [ $? -gt 1 ]; then
  echo "      (F) Failure during run of Core pre script."
  echo "          Check the Core pre script Execution log for details"
  exit 130
fi

# move log to app logs dir
mv "$TMPDIR"/$tmp_log_file "$APP_LOGDIR"


echo "      (I) Completed preparing and executing the core pre script"
return 0;
}


###


# If the app team has a after scirpt, kick it off
exec_post_core_afterscript()
{
echo "==> Preparing to Execute the Core After Script"

after_script_file="$TWH_TOPDIR/install/bin/twh_core_after.sh"
tmp_log_file_name="tmp_core_afterscript.log"


if [ ! -r "$after_script_file" ]; then
  echo "      (I) No Core afterscript found."
  echo "          Exiting function..."
  return 0;
fi


if [ ! -x "$after_script_file" ]; then
  chmod 777 "$after_script_file"
fi

# Fake a couple of env vars for this to run
APP_INSTALL_LIST_FILE="blah"; export APP_INSTALL_LIST_FILE


echo "      (I) Executing Core after script"
echo "          Core Execution log is ($TMPDIR/$tmp_log_file_name)"
"$after_script_file" > "$TMPDIR/$tmp_log_file_name" 2>&1
if [ $? -gt 1 ]; then
  echo "      (F) Failure during run of Core after script."
  echo "          Check the Core after script Execution log for details"
  exit 130
fi

# move log to app logs dir
mv "$TMPDIR"/$tmp_log_file_name "$APP_LOGDIR"


echo "      (I) Completed preparing and executing the core after script"
return 0;
}


###


# Driver function for Component installations
# Tech Notes:
#	We use the NT DWC host as the anchor for all DB work for all
#	DB's (i.e. MD, CDW, and Mart).  So the other local options would
#	be physical objects like copying files.
execute_install_driver()
{

# Pre execution for all options
read_core_comp_list_file;
validate_app_path;
read_app_props_file "$APP_TO_INSTALL" "$APP_MEDIA_DIR";
check_core_tools_installed;
set_app_program_vars;

validate_app_rel_version;
gr_test_db2exec;

# Mark the start of fixpack
echo "*** Starting DB2 fixpack work for ($AVA_CODE) ***" >> "$TMPDIR"/$DB2_INSTALL_LOG
echo "" >> "$TMPDIR"/$DB2_INSTALL_LOG
gr_mark_db2_log_for_install "MARK$AVA_CODE";
  
create_patch_dirs;
create_app_dir_backup;

echo ""
echo ""
echo "==> <<< Executing main driver for application ($AVA_CODE) >>>"
echo ""
echo ""

test_parent_wpack_prereq;
exec_core_prescript;


# Now install each component they requested - this is in order
### Admin server (DWC control server NT host)
if [ $ADMIN_CHECKED -eq $TRUE ]; then
  # For this option we will do all the DB work for all components
  echo ""
  echo "==> Admin component install detected on this host";

  # NT only for this!
  gr_check_interp_by_class $CLASS_WIN;

  move_media_to_prod_tree;
  execute_md_db_backup;

  exec_pre_script;
  if [ "$CDW_INCLUDED" = "TRUE" ]; then exec_cdw_updates; fi
  if [ "$MART_INCLUDED" = "TRUE" ]; then exec_mart_updates; fi
  if [ "$ETL_TAG_INCLUDED" = "TRUE" ]; then 
    exec_pre_etl_script;
    drop_dwc_data_4_update;
    import_etl; 
    exec_ctrl_updates;
  fi
  if [ "$REPORTS_INCLUDED" = "TRUE" ]; then install_reports; fi
  if [ "$I18N_INCLUDED" = "TRUE" ]; then load_app_nls_into_db; fi
fi


### CDW
if [ $CDW_CHECKED -eq $TRUE ]; then
# For later code expansion
  echo ""
  echo "==> CDW component install detected on this host";

  # Check acceptable interps
  gr_check_interp_by_class $CLASS_TYPE_1;

  if [ "$ADMIN_CHECKED" -ne $TRUE ]; then
    move_media_to_prod_tree;
  fi
fi


### Mart DB
if [ $MART_CHECKED -eq $TRUE ]; then
# For later code expansion
  echo ""
  echo "==> Mart component install detected on this host";

  # Check acceptable interps 
  gr_check_interp_by_class $CLASS_TYPE_1;

  if [ "$ADMIN_CHECKED" -ne $TRUE -a "$CDW_CHECKED" -ne $TRUE ]; then
    move_media_to_prod_tree;
  fi
fi


### RPI
if [ $RPI_CHECKED -eq $TRUE ]; then
  echo ""
  echo "==> RPI component install detected on this host";

  # Check acceptable interps
  gr_check_interp_by_class $CLASS_TYPE_2;

  if [ "$ADMIN_CHECKED" -ne $TRUE -a "$CDW_CHECKED" -ne $TRUE -a "$MART_CHECKED" -ne $TRUE  ]; then
    build_rpi_dirs;
    move_media_to_prod_tree;
  fi

  if [ "$I18N_INCLUDED" = "TRUE" ]; then install_lang_files; fi
fi


# Post execution for all
echo ""
exec_after_script;
exec_post_core_afterscript;


return 0;
}


###


#
post_details()
{
echo ""
echo "==> TEDW Warehouse Pack Application Patch Installation Details:"
echo ""
echo "       Application:               $APP_DISPLAY_NAME"
echo "       Installed in:              $APP_TOPDIR"
echo ""
echo "       Installation Properties:"
echo "          AVA Code:               $AVA_CODE"
echo "          Required Core Version:  $TWH_CORE_PREREQ_VERSION"
echo "          App Patch Version:      $APP_REL_VERSION"
echo "          Pre Script:             $PRE_SCRIPT"
echo "          Pre ETL Script:         $PRE_ETL_SCRIPT"
echo "          After Script:           $AFTER_SCRIPT"
echo "          Datamart Schmea name:   $APP_DATAMART_SCHEMA"
echo "          CDW Included:           $CDW_INCLUDED"
echo "          Docs Included:          $DOCS_INCLUDED"
echo "          ETL SQL Included:       $ETL_SQL_INCLUDED"
echo "          ETL Tag Included:       $ETL_TAG_INCLUDED"
echo "          I18n Included:          $I18N_INCLUDED"
echo "          Mart Included:          $MART_INCLUDED"
echo "          Reports Included:       $REPORTS_INCLUDED"
echo "          Pre Required WPACK:     $WPACK_PREREQ_AVA_NAME"
if [ -n "$TWH_KEEP_TMP_FILES_TRUE" ]; then
  echo "          <Keeping tmp files>"
fi
echo ""
echo "       (I) TEDW Warehouse Pack Details completed"
echo ""
return 0;
}


###


# Clean up
clean_up()
{
tmp_conf_file="tmp_$TWH_CONFIG_FILE_NAME"

echo ""
echo "==> Starting post-installer cleanup"

# Remove the password from the config file
cat "$TWH_CONFIG_FILE" | sed "s/DB2PASS=.*/DB2PASS=/" > "$TMPDIR/$tmp_conf_file"
rm -f "$TWH_CONFIG_FILE"
mv "$TMPDIR/$tmp_conf_file" "$TWH_CONFIG_FILE"
cp -f "$TWH_CONFIG_FILE" "$APP_LOGDIR"/$TWH_CONFIG_FILE_NAME.done

# if in tmpdir get rid of it, otherwise keep it
cfg_path=`dirname "$TWH_CONFIG_FILE" `
if [ "$cfg_path" = "$TMPDIR" ]; then
  rm -f "$TWH_CONFIG_FILE"
fi


# Safety net - Remove the connect strings from any remaining files
# If directed, clean logs and tmp files
if [ -z "$TWH_KEEP_TMP_FILES_TRUE" ]; then
  echo "      (I) Removing unnecessary tmp files"
  rm -f "$TMPDIR"/tmp_twh*.sql
  rm -f "$TMPDIR"/tmp_${AVA_CODE}*.sql
  rm -f "$APP_LOGDIR"/tmp_twh*.sql
  rm -f "$APP_LOGDIR"/tmp_${AVA_CODE}*.sql
else
  echo "      (I) Cleaning tmp files"
  # if any file left here, clean them and move them
  # Remove the connect strings from files and move them to app logdir
  ls "$TMPDIR"/tmp_twh*.sql  2> /dev/null  > "$TMPDIR"/sqlfiles.txt
  ls "$TMPDIR"/tmp_${AVA_CODE}*.sql  2> /dev/null  >> "$TMPDIR"/sqlfiles.txt;
  while :
  do
    read sql_file
    if [ "$sql_file" = "" ]; then
        break
    fi
    gr_clean_conn_string "$sql_file" "$APP_LOGDIR";
  done < "$TMPDIR"/sqlfiles.txt
  rm -f "$TMPDIR"/sqlfiles.txt

  # Move any remaining file types
  blah=`cp -f "$TMPDIR"/tmp_twh* "$APP_LOGDIR" 2>&1 `
  blah=`cp -f "$TMPDIR"/tmp_${AVA_CODE}* "$APP_LOGDIR" 2>&1 `
fi


# copy over the properties file
blah=`cp -f "$APP_MEDIA_DIR"/$APP_INSTALL_PROPS_FILE "$APP_LOGDIR" 2>&1`
if [ $? -ne 0 ]; then
  echo "      (F) Failed trying to copy the App properties file"
  exit 130
fi


# make a copy of the  DB2 SQL install log for this installation
cp -f "$TMPDIR"/$DB2_INSTALL_LOG "$APP_LOGDIR"


# make a copy of this install
cp -f "$TMPDIR/twh_install_wpack_patcher.log" "$APP_LOGDIR"

# Make a copy of the DWC report
if [ -r "$TMPDIR"/${AVA_CODE}_dwc_upgrade_relationships.report ]; then
  cp -f "$TMPDIR"/${AVA_CODE}_dwc_upgrade_relationships.report  "$APP_LOGDIR" 
fi


echo "      (I) Post-installer cleanup completed"
return 0;
}



###


#
prep_exit()
{
# Set the success tag file for this app
touch "$APP_VER_TAG_FILE"


echo ""
echo "==> TEDW Warehouse Pack Patch Installation Successfully Completed!!!"


# make a copy of this install
cp -f "$TMPDIR/twh_install_wpack_patcher.log" "$APP_LOGDIR"

# Turn off the Core context for the Generic routines
unset TWH_APP_PATCH_CONTEXT


return 0;
}


###############################################################
############## Driver Code Section ############################
###############################################################

### Set Program Vars
# Non configurable vars
# Required in all files using Generic Routines (gr_)
ARGS_RUN="$@"
ARGS_IN=$*
ARGS_NO=$#
PROGRAM=$0
gen_file_and_path=`echo $0 | sed 's/\\\\/\//g' `	# NT fix
PROGNAME=`basename "$gen_file_and_path"`
TRUE=1
FALSE=0
CUSTOM_INSTALL=$FALSE
ADMIN_CHECKED=$FALSE
RPI_CHECKED=$FALSE
CDW_CHECKED=$FALSE
MART_CHECKED=$FALSE


############
### Main ###
############

# Bring in the generic routines
include_gen_routines;

# Loop flags
args_found=0
conf_val_found=$FALSE

while [ $# -ne 0 ]; do
  # Now look for the command line option
  case $1 in

        # Application Patch Config file
        --config|--CONFIG|--Config|-C|-c)
            # Set flag for value found
            conf_val_found=$TRUE
            shift
            TWH_CONFIG_FILE="$1"; export TWH_CONFIG_FILE
		if [ ! -r "$TWH_CONFIG_FILE" ]; then
		  echo "      (F) Value not valid for config file name ($TWH_CONFIG_FILE)"
		  exit 131
		fi
            shift
            args_found=`expr $args_found + 2`
            continue
            ;;

        # Version Info
        --version|--VERSION|--Version|-V|-v)
            echo ""
            echo "  $PROGNAME"
            echo "  Version 1.1.1"
            echo "  A Tivoli Enterprise Data Warehouse utility"
		echo "  IBM Software"
            echo ""
            exit 0
            ;;

        # Help Info
        --help|--HELP|-Help|-H|-h)
            display_help;
            exit 0
            ;;

        # Display the return code messages
        --retcode|--RETCODE|--Retcode|-R|-r)
            gr_display_generic_header "TEDW Warehouse Pack Application Patch installer";
            display_return_code_messages;
            exit 0
            ;;

        # Run in Debug
        --debug|--DEBUG|--Debug|-D|-d)
            DEBUG=defined
            set_debug_mode;
		shift
		continue
            ;;

	# List installed versions - Can't use '-l' - bug on NT Bash engine.
	--listversions|--LISTVERSIONS|--Listversions|-L|-l)
		echo ""
		echo "    <<< Listing installed product, patch, and e-fix versions only >>>"
            echo ""
	      LIST_INSTALLED=defined; export LIST_INSTALLED
		BYPASS_PASSWORD_REQUIRED=true; export BYPASS_PASSWORD_REQUIRED
	      shift
	      continue
	      ;;

        # Error - not valid option
        -*)
            echo "      (F) Unknown Command option ($1)"
            echo "          < Displaying Help >"
              echo ""
            display_help;
            exit 132
            ;;

        # Default
        *)
            echo "      (F) Unknown option ($1)"
            echo "          < Displaying Help >"
              echo ""
            display_help;
            exit 133
            ;;
  esac
done


# Main routine execution
gr_process_args 2 $args_found;
gr_display_generic_header "TEDW Warehouse Pack Application Patch installer";
if [ -z "$LIST_INSTALLED" ]; then display_warning; fi
gr_set_debug_mode;
gr_set_interp;
gr_read_config_file;
validate_basic_install
gr_set_program_vars;
set_private_program_vars;
gr_extend_path;
gr_check_user;


# Execute main routines
if [ -n "$LIST_INSTALLED" ]; then
  exec_list_installed;
else
  execute_install_driver;
:;
fi


clean_up;
post_details;
prep_exit;


# Set exit value
if [ $WARNING_FLAG -eq $TRUE ]; then
  exit 1					# Warnings detected during run
else
  exit 0
fi
